	/***********************************************************************************************
	*       TP sur  EID210 / Rseau CAN - VMD  (Vhicule Multiplex Didactique)					
	************************************************************************************************
	*  	TP n 3: Vrifier le fonctionnement d'un bloc optique
	*----------------------------------------------------------------------------------------------			
	*   CAHIER DES CHARGES :					      						
	*  *********************						      						
	* On souhaite en enchanement cyclique des diffrents tats du bloc optique avant droit
	*	(Rien, Veilleuse, Veilleuse+Stop, Veilleuse+Phare etc) et un fonctionnement du clignoteur.
	*  On ralisera en plus des fonctions de controle:								
	*	- On vrifiera que le module command renvoi bien une trame d'acquittement.
	*	- On vrifiera (par fonction logicielle) que les lampes commandes sont effectivement 
	*		allumes 
	*	- On affichera quelles sont les lampes concernes ainsi que le rsultat du test.
	*	-  Le clignotement (du clignotant) sera indpendant de la permutation des autres lampes. 
	* On impose les priodes suivantes, dans l'ordre de priorit dcroissante:
      *    	- priode de permutation des lampes 3,5 S,
      *   	- priode de commutation du clignoteur 1,6 S,
 	*  Ces priodes devront pouvoir tre modifies facilement.
	*	-  Le programme devra permettre un changement ais de bloc cible.
 	*----------------------------------------------------------------------------------------------	
	*  NOM du FICHIER :  CAN_VMD_TP3.C			       							
	* ***************** 													
	*************************************************************************************************/
// Fichiers  inclure
//********************
#include <stdio.h>
#include "Structures_Donnees.h"
#include "cpu_reg.h"
#include "eid210_reg.h"
#include "Can_vmd.h"

// Pour les temporisations
#define Tempo_Feux 30	// En dizime de seconde -> 3 S
#define Tempo_Clignot 16	// En dizime de seconde -> 1,6 S
#define Tempo_Att_Rep 40	// Attente rponse en dizime de seconde ->4 S
// Pour le codage des tats
#define Etat_aucune 0
#define Etat_veilleuse 1
#define Etat_code 2
#define Etat_phare 3
// Dclaration des variables
//---------------------------
// Pour les variables images
union byte_bits Image_Feux,Image_Etat_Feux,Indicateurs;
#define Valeur_Feux Image_Feux.valeur	// Pour un accs port complet
#define Veilleuse Image_Feux.bit.b0 
#define Code Image_Feux.bit.b1
#define Phare Image_Feux.bit.b2
#define Clignot Image_Feux.bit.b3
#define S_Veilleuse Image_Etat_Feux.bit.b4 // Status veilleuse
#define S_Code Image_Etat_Feux.bit.b5
#define S_Phare Image_Etat_Feux.bit.b6
#define S_Clignot Image_Etat_Feux.bit.b7
// Pour les indicateurs divers
#define I_Att_Rep_Acquit Indicateurs.bit.b0
#define I_Fin_Tempo_Feux Indicateurs.bit.b1
#define I_Fin_Tempo_Clignot Indicateurs.bit.b2
#define I_Fin_Tempo_Control Indicateurs.bit.b3
#define I_Fin_Tempo_Att_Rep Indicateurs.bit.b4
#define I_Att_Rep_Interrog Indicateurs.bit.b5
#define I_Autorise_Emis_Mes Indicateurs.bit.b6
#define I_Message_Pb_Affiche Indicateurs.bit.b7
// Dclaration des trames
Trame Trame_Recue;
Trame Trame_Envoyee;
Trame T_IM_Feux;   // Trame de type "Input Message" pour commande module 4 Sorties de puissance
Trame T_IRM_Feux;  // Trame de type "Information Request Message" pour interroger les tats lampes 
// Pour la comparaison des identificateurs Trame envoye <-> Trame reue 
#define Ident_Trame_Envoyee Trame_Envoyee.ident.extend.identificateur.ident
#define Ident_Trame_Recue Trame_Recue.ident.extend.identificateur.ident

// Pour les temporisations
WORD Compteur_Passage,Compteur_dS; // dS -> dizime de Seconde
WORD Valeur_Fin_Tempo_Feux,Valeur_Fin_Tempo_Clignot,Valeur_Fin_Tempo_Att_Rep;
// Pour le diagramme des tats
unsigned char Etat;
// Pour controle communication
int Cptr_TimeOut,Temp;


//  Fonction d'interruption "Base de Temps"
//=====================================
void irq_bt()
// Fonction excute toute les 10 mS
{Compteur_Passage++;
if(Compteur_Passage==10)  // Un 1/10 Seconde s'est coule
	{Compteur_Passage=0;
	 Compteur_dS++;
	 if(Compteur_dS==Valeur_Fin_Tempo_Feux)
		{I_Fin_Tempo_Feux = 1;
		 Valeur_Fin_Tempo_Feux = Compteur_dS + Tempo_Feux;}
	 if(Compteur_dS==Valeur_Fin_Tempo_Clignot)
		{I_Fin_Tempo_Clignot = 1;
		 Valeur_Fin_Tempo_Clignot = Compteur_dS + Tempo_Clignot;}
 	 if(Compteur_dS==Valeur_Fin_Tempo_Att_Rep)
		{I_Fin_Tempo_Att_Rep = 1;}
	}
} // Fin de la fonction d'interruption

//==========================
//  FONCTION PRINCIPALE
//==========================
main()
{
// Initialisations
//*****************
clsscr();
//  Dfinition des trames pour activer ou lire un bloc optique 
// D'aprs doc SJA1000 et  doc MCP25050 pages 22 (fonction "Write Register") et 37 (Adresse GPPIN)
// Pour trame de commande  -> IM  
T_IM_Feux.trame_info.registre=0x00;
T_IM_Feux.trame_info.champ.extend=1; // On travaille en mode tendu
T_IM_Feux.trame_info.champ.dlc=0x03; // Il y aura 3 donnes de 8 bits (3 octets envoys)
T_IM_Feux.ident.extend.identificateur.ident=Ident_T_IM_FVD; // l'identificateur Feux aVant Gauche
								     // Voir dfinitions dans CAN_VMD.h
// Pour trame interrogative ->  IRM  (Information Request Trame)
T_IRM_Feux.trame_info.registre=0x00;
T_IRM_Feux.trame_info.champ.extend=1;
T_IRM_Feux.trame_info.champ.dlc=0x01;
T_IRM_Feux.trame_info.champ.rtr=1;
T_IRM_Feux.ident.extend.identificateur.ident=Ident_T_IRM_FVD; // Voir dfinitions dans CAN_VMD.h			

/* Initialisation DU SJA1000 de la carte ATON-Systemes" sur  */
Init_Aton_CAN();
// Envoi trame pour dfinir de la direction des entres sorties
T_IM_Feux.data[0]=0x1F;	// premire donne -> "Adresse" du registre concerne -> GPDDR
T_IM_Feux.data[1]=0x7F;	// deuxime donne -> "Masque" ->  Voir Doc page 16
T_IM_Feux.data[2]=0xF0;	// troisime donne -> "Valeur" -> les sorties sur 4 bits de poids faibles
I_Message_Pb_Affiche=0;
do {Ecrire_Trame(T_IM_Feux);// Envoyer une premire trame sur rseau CAN 
	 Cptr_TimeOut=0;
	 do{Cptr_TimeOut++;}while((Lire_Trame(&Trame_Recue)==0)&&(Cptr_TimeOut<200));
       if(Ident_Trame_Recue!=Ident_T_AIM_FVD)Cptr_TimeOut=200; // Test si identificateur correct
	 if(Cptr_TimeOut==200)
		{if(I_Message_Pb_Affiche==0)
			{I_Message_Pb_Affiche=1;
			 gotoxy(2,10);
		 	 printf(" Pas de reponse a la trame de commande en initialisation \n");
		 	 printf("  Verifier si alimentation 12 V  est OK \n");}
		for(Temp=0;Temp<100000;Temp++);} // Pour attendre un peu! 
    }while(Cptr_TimeOut==200);
clsscr();
// Pour l'ensemble des indicateurs
Indicateurs.valeur=0;
Etat = Etat_aucune;
Valeur_Feux=0x00;
// On envoie sur le bus la premire trame -> tat initial des sorties (qui sont initialises  0
// Initialiser les sorties  0
T_IM_Feux.data[0]=0x1E;	// premire donne -> "Adresse" du registre concerne (GPLAT dfinit l'tat des sorties) 03h+1Ch = 1Eh
T_IM_Feux.data[1]=0x0F;	// deuxime donne -> "Masque" -> les sorties sont sur les 4 bits de poids faibles
T_IM_Feux.data[2]=0x00;	// troisime donne -> "Valeur" -> au dpart toutes les sorties sont  0 (lampes teintes)

Ecrire_Trame(T_IM_Feux);
Trame_Envoyee = T_IM_Feux;
I_Att_Rep_Acquit=1;
// Pour base de temps et temporisations
//***************************************
SetVect(96,&irq_bt);	// mise en place de l'autovecteur 
PITR = 0x0048;		// Une interruption toutes les 10 millisecondes 
PICR = 0x0760; 		//  96 = 60H
Compteur_Passage = 0,Compteur_dS = 0;
Valeur_Fin_Tempo_Feux = Tempo_Feux;
Valeur_Fin_Tempo_Clignot = Tempo_Clignot;
Valeur_Fin_Tempo_Att_Rep = Tempo_Att_Rep;
I_Autorise_Emis_Mes=1;
//   Afficher titre
gotoxy(1,1);
printf("    TP  n  3    ACTIVER FEUX ET CONTROLER ETAT AMPOULES FVD\n");
printf("    *********************************************************\n");
// Boucle principale
//*******************
while(1)
	{
	if (1==Lire_Trame(&Trame_Recue)) //Si trame reue, la fonction retourne 1
		{ // On vient de recevoir une trame en rponse
		 gotoxy(1,3);  // Pour effacer message d'alerte de non reponse module adresse
		 printf("                                                                               \n");
  	         	if(I_Att_Rep_Acquit)
			{ // On attendait une trame d'acquittement suite  l'envoi d'une commande feux
			  I_Att_Rep_Acquit=0;
			  I_Autorise_Emis_Mes=1;
			  gotoxy(1,8);
			  printf("    Trame d'acquittement suite  une 'IM' (Input Message)\n");
			  Affiche_Trame(Trame_Recue);
			  // On peut envoyer trame interrogative afin de vrifier l'tat des feux
			  Ecrire_Trame(T_IRM_Feux); 
		  	 Valeur_Fin_Tempo_Att_Rep = Compteur_dS+Tempo_Att_Rep;
			 I_Fin_Tempo_Att_Rep=0;
			  gotoxy(1,12);
		  	  printf("    Trame interrogative envoyee -> 'IRM' (Information  Request Message) \n");
			  printf("    En vue de tester le bon fonctionnement des feux \n");
		  	  Trame_Envoyee = T_IRM_Feux;
	 		  Affiche_Trame(Trame_Envoyee);
		  	  I_Att_Rep_Interrog=1;
			}
		   else if(I_Att_Rep_Interrog)
				{// On attendait une trame de rponse  une interrogation
				 I_Att_Rep_Interrog=0;
				 I_Autorise_Emis_Mes=1;
			  	 gotoxy(1,16);
			  	 printf("    Trame reponse a une interrogation -> 'OM' (Output Message) \n");
			  	 Affiche_Trame(Trame_Recue);

				// Analyse de l'tat des ampoules et affichage rsultat diagnostique
					 Image_Etat_Feux.valeur=Trame_Recue.data[0];
					 if(Veilleuse==1 && S_Veilleuse==0)
						{gotoxy(1,20),printf("!!     Probleme sur Veilleuse  \n");}
					 if(Veilleuse==1 && S_Veilleuse==1)
						{gotoxy(1,20),printf("!!     Veilleuse   OK                  \n");}
					 if(Code==1 && S_Code==0)
						{gotoxy(1,21),printf("!!     Probleme sur Code         \n");}
					 if(Code==1 && S_Code==1)
						{gotoxy(1,21),printf("!!     Code   OK                       \n");}
					 if(Phare==1 && S_Phare==0)
						{gotoxy(1,22),printf("!!     Probleme sur Phare   \n");}
					 if(Phare==1 && S_Phare==1)
						{gotoxy(1,22),printf("!!     Phare   OK                   \n");}
					 if(Clignot==1 && S_Clignot==0)
						{gotoxy(1,23),printf("!!     Probleme sur Clignotant      \n");}
					 if(Clignot==1 && S_Clignot==1)
						{gotoxy(1,23),printf("!!     Clignotant   OK                \n");}
				}
		}
 		 if(I_Fin_Tempo_Feux)
			{I_Fin_Tempo_Feux=0;
			// On passe  l'tat suivant 
			switch(Etat)
				{case Etat_aucune : 	{Etat=Etat_veilleuse;
							   	 Veilleuse =1; }
				 break;
				 case Etat_veilleuse : 	{Etat=Etat_code;
								 Code=1;}
				 break;
				 case Etat_code : 	{Etat=Etat_phare;
								 Code=0,Phare=1;}
			 break;
				 case Etat_phare : 	{Etat=Etat_aucune;
								 Veilleuse=0,Phare=0;}
				 break;}
			// On envoie la trame de commande avec les tats mis  jour
			 T_IM_Feux.data[2]=Valeur_Feux;
			if(I_Autorise_Emis_Mes)
		 	 	{Ecrire_Trame(T_IM_Feux);  // Envoyer trame de commande sur rseau CAN
			 	gotoxy(1,5);
			 	printf("    Trame de commande  -> 'IM' (Input Message) \n");
			 	Trame_Envoyee = T_IM_Feux;
			 	Affiche_Trame(Trame_Envoyee);
		 	 	I_Att_Rep_Acquit=1;
			 	I_Autorise_Emis_Mes=0;
				Valeur_Fin_Tempo_Att_Rep = Compteur_dS+Tempo_Att_Rep;
				I_Fin_Tempo_Att_Rep=0;}}
		 else if(I_Fin_Tempo_Clignot)
			{
			 I_Fin_Tempo_Clignot=0;
			 // On change l'tat du clignotant
			 Clignot=~Clignot;
			// On envoie la trame de commande
			T_IM_Feux.data[2]=Valeur_Feux;
		 	 if(I_Autorise_Emis_Mes)
				{Ecrire_Trame(T_IM_Feux);  // Envoyer trame de commande sur rseau CAN
			 	Trame_Envoyee = T_IM_Feux;
		 	 	I_Att_Rep_Acquit=1;
			 	I_Autorise_Emis_Mes=0;
				Valeur_Fin_Tempo_Att_Rep = Compteur_dS+Tempo_Att_Rep;
				I_Fin_Tempo_Att_Rep=0;}}
		 else if(I_Fin_Tempo_Att_Rep)
			// Cela fait trop longtemps que l'on attend une reponse!
			 {clsscr();
			gotoxy(1,1);
			printf("    TP  n  3    ACTIVER FEUX ET CONTROLER ETAT AMPOULES   \n");
			printf("    ********************************************************* \n");
			printf("!!        Le module adresse ne  repond plus         !!\n");
			Ecrire_Trame(T_IRM_Feux); // On ritre une interrogation
			Valeur_Fin_Tempo_Att_Rep = Compteur_dS+Tempo_Att_Rep; // On rarme la temporisation 
			I_Fin_Tempo_Att_Rep=0; } 
	} // FIN de la boucle principale
} // FIN de la fonction principale