/************************************************************************************************
	Ralisation du VMD en mode automatique:
	On souhaite que les modules nous envoies automatiquement une trame 
	lors d'un changement d'tat de leurs sortie.
	Structure du programme:
		-Partie1: Configuration des modules:
			Definition des E/S
			Imposer les tats d'origine
			Configurer les modules en envoie automatique
		-Partie2: Programme principal
			Recevoir les trames presentes sur le can et agir sur les modules
			Re-Configurer les Modules pour prendre en compte les changement d'tat
			Allumer Les Feux en fonction du commodo
			Gerer le clignottement
			Afficher un compte rendu de l'tat des Modules

*************************************************************************************************/
// Fichiers  inclure
//********************
#include <stdio.h>
#include "Structures_Donnees.h"
#include "cpu_reg.h"
#include "eid210_reg.h"
#include "Can_vmd.h"

// Definition des identifiants des message automatique
#define Ident_T_MesAuto_FRG 0x0F400000	// Feux aRrire Gauche 
#define Ident_T_MesAuto_FRD 0x0FC00000	// Feux aRrire Droit
#define Ident_T_MesAuto_FVG 0x0E400000	// Feux aVant Gauche 
#define Ident_T_MesAuto_FVD 0x0EC00000	// Feux aVant Droit
#define Ident_T_MesAuto_ComLum 0x05400000	// Commodo Lumiere

//Structure d'etat programme afin de savoir les trames attendus:
typedef struct{
unsigned char Env_Ordre_FRG;
unsigned char Env_Cfg_FRG;
unsigned char ACK_IM_FRG_Attendu;
unsigned char ACK_Cfg_FRG_Attendu;

unsigned char Env_Ordre_FRD;
unsigned char Env_Cfg_FRD;
unsigned char ACK_IM_FRD_Attendu;
unsigned char ACK_Cfg_FRD_Attendu;

unsigned char Env_Ordre_FVG;
unsigned char Env_Cfg_FVG;
unsigned char ACK_IM_FVG_Attendu;
unsigned char ACK_Cfg_FVG_Attendu;

unsigned char Env_Ordre_FVD;
unsigned char Env_Cfg_FVD;
unsigned char ACK_Cfg_FVD_Attendu;
unsigned char ACK_IM_FVD_Attendu;

unsigned char ACK_Cfg_ComLum_Attendu;
unsigned char Env_Cfg_ComLum;

unsigned char Affichage;
unsigned char Affichage_Com_Feux;
unsigned char Affichage_FRG;
unsigned char Affichage_FRD;
unsigned char Affichage_FVG;
unsigned char Affichage_FVD;

unsigned char Fin;
}ETAT_PROG;

//prototype des sous fonctions: 
char Cfg_Module(Trame T_Envoie,double ACK_Attendu); //configuration des Modules
void irq_BDT(void); //Fonction d'interruption temporelle

//definition des variables de temporisation
int Base_Temp=500;// dclaration et initialisation de la base de temps gnral:5s => 500*10ms(cfg PITR)
int Base_Temps_Clignot=50; // Frequence de clignotement  0.5s
int compteur; // compteur interruption
char Flag_Clignot;

/*************************************************************************************************/
/****					Programme Principal					*******************/
/*************************************************************************************************/
main()
{
/*Definition des variables necessaires*/
Trame Trame_Recue;
Trame T_IM_Modules;	/*Trame qui va servir pour configurer tout les modules*/
Trame T_IM_Feux;		/*trame de commande a destination des Feux*/

/*Mise en place de la structure sous le nom Flag*/
ETAT_PROG Flag;

/*Memorisation de l'tat des feux*/
unsigned char Valeur_FRG_Mem;
unsigned char Valeur_FRD_Mem;
unsigned char Valeur_FVG_Mem;
unsigned char Valeur_FVD_Mem;

/* Initialisation DU SJA1000 de la carte ATON-Systemes sur le bus PC104  */
Init_Aton_CAN();
clsscr();

/*Configuration des variables*/
compteur=0;
Valeur_Status_FRG=0xf0;
Valeur_Status_FRD=0xf0;
Valeur_Status_FVG=0xf0;
Valeur_Status_FVD=0xf0;
Valeur_FRG=0xf0;
Valeur_FRD=0xf0;
Valeur_FVG=0xf0;
Valeur_FVD=0xf0;

/*configuration des trame definie*/
//1-Feux:
T_IM_Feux.trame_info.registre=0x00;
T_IM_Feux.trame_info.champ.extend=1; // On travaille en mode tendu
T_IM_Feux.trame_info.champ.dlc=0x03; // Il y aura 3 donnes de 8 bits (3 octets envoys)
T_IM_Feux.ident.extend.identificateur.ident=0x00; //on mettra l'identifiant plus tard
T_IM_Feux.data[0]=0x1E;	//Acceder au registre GPLAT
T_IM_Feux.data[1]=0x0F;	//Mask=> Seul les 4 bits de poid faibles sont concerns
T_IM_Feux.data[2]=0x00;	//On verra la valeur plus tard
//2-Configuration
T_IM_Modules.trame_info.registre=0x00;
T_IM_Modules.trame_info.champ.extend=1; // On travaille en mode tendu
T_IM_Modules.trame_info.champ.dlc=0x03; // Il y aura 3 donnes de 8 bits (3 octets envoys)
T_IM_Modules.ident.extend.identificateur.ident=0x00; //on mettra l'identifiant plus tard

/*1ere partie: */

/*Definition des E/S pour les modules*/
T_IM_Modules.data[0]=0x1F;	//Acceder au registre GPPDR
T_IM_Modules.data[1]=0xFF;	//Mask=> tous les bits sont concerns
T_IM_Modules.data[2]=0xF0;	//4 bits de poids fort en entres, 4 de poids faibles en sortie

gotoxy(1,1);
printf("Definition des entres sortie:\n");

gotoxy(2,2);
printf("Feux Arriere Gauche:       \n");
T_IM_Modules.ident.extend.identificateur.ident=Ident_T_IM_FRG;
do{
	gotoxy(2,3);
   }while(Cfg_Module(T_IM_Modules,Ident_T_AIM_FRG)!=1);

gotoxy(2,4);
printf("Feux Arriere Droit:       \n");
T_IM_Modules.ident.extend.identificateur.ident=Ident_T_IM_FRD;
do{
	gotoxy(2,5);
   }while(Cfg_Module(T_IM_Modules,Ident_T_AIM_FRD)!=1);

gotoxy(2,6);
printf("Feux Avant Gauche:       \n");
T_IM_Modules.ident.extend.identificateur.ident=Ident_T_IM_FVG;
do{
	gotoxy(2,7);
   }while(Cfg_Module(T_IM_Modules,Ident_T_AIM_FVG)!=1);

gotoxy(2,8);
printf("Feux Avant Droit:       \n");
T_IM_Modules.ident.extend.identificateur.ident=Ident_T_IM_FVD;
do{
	gotoxy(2,9);
   }while(Cfg_Module(T_IM_Modules,Ident_T_AIM_FVD)!=1);

gotoxy(2,10);
printf("Commodo Lumiere:       \n");
T_IM_Modules.ident.extend.identificateur.ident=Ident_T_IM_Commodo_Feux;
T_IM_Modules.data[2]=0xFF;	//!! sur commodo les 8 bits sont en entres
do{
	gotoxy(2,11);
   }while(Cfg_Module(T_IM_Modules,Ident_T_AIM_Commodo_Feux)!=1);

clsscr();

/*Imposer les tats des sorties sur les Feux*/
T_IM_Modules.data[0]=0x1E;	//Acceder au registre GPLAT
T_IM_Modules.data[1]=0x0F;	//Mask=> Seul les bits de poids faibles sont concernes
T_IM_Modules.data[2]=0x00;	//Et on veut les etteindre

gotoxy(1,1);
printf("Imposer les Etat des Feux:\n");

gotoxy(2,2);
printf("Feux Arriere Gauche:       \n");
T_IM_Modules.ident.extend.identificateur.ident=Ident_T_IM_FRG;
do{
	gotoxy(2,3);
   }while(Cfg_Module(T_IM_Modules,Ident_T_AIM_FRG)!=1);

gotoxy(2,4);
printf("Feux Arriere Droit:       \n");
T_IM_Modules.ident.extend.identificateur.ident=Ident_T_IM_FRD;
do{
	gotoxy(2,5);
   }while(Cfg_Module(T_IM_Modules,Ident_T_AIM_FRD)!=1);

gotoxy(2,6);
printf("Feux Avant Gauche:       \n");
T_IM_Modules.ident.extend.identificateur.ident=Ident_T_IM_FVG;
do{
	gotoxy(2,7);
   }while(Cfg_Module(T_IM_Modules,Ident_T_AIM_FVG)!=1);

gotoxy(2,8);
printf("Feux Avant Droit:       \n");
T_IM_Modules.ident.extend.identificateur.ident=Ident_T_IM_FVD;
do{
	gotoxy(2,9);
   }while(Cfg_Module(T_IM_Modules,Ident_T_AIM_FVD)!=1);

clsscr();

/*Configuration en mode automatique*/
T_IM_Modules.data[0]=0x1C;	//Acceder au registre IOINTEN
T_IM_Modules.data[1]=0xF0;	//Mask=> Seul les bits configurer en entres peuvent fonctionner
T_IM_Modules.data[2]=0xF0;	//On active donc les 4 bits de poids fort

gotoxy(1,1);
printf("Mise en place des messages automatique:\n");

gotoxy(2,2);
printf("Feux Arriere Gauche:       \n");
T_IM_Modules.ident.extend.identificateur.ident=Ident_T_IM_FRG;
do{
	gotoxy(2,3);
   }while(Cfg_Module(T_IM_Modules,Ident_T_AIM_FRG)!=1);

gotoxy(2,4);
printf("Feux Arriere Droit:       \n");
T_IM_Modules.ident.extend.identificateur.ident=Ident_T_IM_FRD;
do{
	gotoxy(2,5);
   }while(Cfg_Module(T_IM_Modules,Ident_T_AIM_FRD)!=1);

gotoxy(2,6);
printf("Feux Avant Gauche:       \n");
T_IM_Modules.ident.extend.identificateur.ident=Ident_T_IM_FVG;
do{
	gotoxy(2,7);
   }while(Cfg_Module(T_IM_Modules,Ident_T_AIM_FVG)!=1);

gotoxy(2,8);
printf("Feux Avant Droit:       \n");
T_IM_Modules.ident.extend.identificateur.ident=Ident_T_IM_FVD;
do{
	gotoxy(2,9);
   }while(Cfg_Module(T_IM_Modules,Ident_T_AIM_FVD)!=1);

gotoxy(2,10);
printf("Commodo Lumiere:       \n");
T_IM_Modules.ident.extend.identificateur.ident=Ident_T_IM_Commodo_Feux;//!! commodo Feux=8Entres
T_IM_Modules.data[1]=0xFF;	//Mask=> Seul les bits configurer en entres peuvent fonctionner
T_IM_Modules.data[2]=0xFF;	//On active donc les 8 bits
do{
	gotoxy(2,11);
   }while(Cfg_Module(T_IM_Modules,Ident_T_AIM_Commodo_Feux)!=1);

clsscr();

/*Il faut maintenant imposer les seuile de basculement qui dclancheronts les envois*/
T_IM_Modules.data[0]=0x1D;	//Acceder au registre IOINTPO
T_IM_Modules.data[1]=0xF0;	//Mask=> Seul les bits configurer en entres peuvent fonctionner
//Sur les feux, on a tout etteind=> les status sont donc  1=>il faudra prendre en compte un font descendant donc un 0 logique
T_IM_Modules.data[2]=0x00;

gotoxy(1,1);
printf("Mise en place des messages automatique:\n");

gotoxy(2,2);
printf("Feux Arriere Gauche:       \n");
T_IM_Modules.ident.extend.identificateur.ident=Ident_T_IM_FRG;
do{
	gotoxy(2,3);
   }while(Cfg_Module(T_IM_Modules,Ident_T_AIM_FRG)!=1);

gotoxy(2,4);
printf("Feux Arriere Droit:       \n");
T_IM_Modules.ident.extend.identificateur.ident=Ident_T_IM_FRD;
do{
	gotoxy(2,5);
   }while(Cfg_Module(T_IM_Modules,Ident_T_AIM_FRD)!=1);

gotoxy(2,6);
printf("Feux Avant Gauche:       \n");
T_IM_Modules.ident.extend.identificateur.ident=Ident_T_IM_FVG;
do{
	gotoxy(2,7);
   }while(Cfg_Module(T_IM_Modules,Ident_T_AIM_FVG)!=1);

gotoxy(2,8);
printf("Feux Avant Droit:       \n");
T_IM_Modules.ident.extend.identificateur.ident=Ident_T_IM_FVD;
do{
	gotoxy(2,9);
   }while(Cfg_Module(T_IM_Modules,Ident_T_AIM_FVD)!=1);

gotoxy(2,10);
printf("Commodo Lumiere:       \n");
T_IM_Modules.ident.extend.identificateur.ident=Ident_T_IM_Commodo_Feux;//!! commodo Feux=8Entres
T_IM_Modules.data[1]=0xFF;	//Mask=> Seul les bits configurer en entres peuvent fonctionner
//Sur le commodo, il y a des resistance de pull up => les etats  sont donc  1=>il faudra prendre en compte un font descendant donc un 0 logique
T_IM_Modules.data[2]=0x00;	

do{
	gotoxy(2,11);
   }while(Cfg_Module(T_IM_Modules,Ident_T_AIM_Commodo_Feux)!=1);

printf("\n\n\nTout les modules ont biens repondus!\n Demarrage du programme\n");
clsscr();

//Configuration de la liaison srie
SCCR0 = 9;		// Pour la vitesse de transmission de 57600 bauds
SCCR1 = 0x000C;	// Pour valider l'mission et la rception
//installation vecteur interruption temporaire 10ms
PITR = 0x0050;		// Une interruption toutes les 10 millisecondes 
PICR = 0x0760; 		//  96 = 60H
SetVect(96,&irq_BDT); //installation vecteur

/**************************************/
/****Boucle sans Fin*******************/
/**************************************/
do{

/*Gestion des Trames Recu**********************************************************************************************/
   if(Lire_Trame(&Trame_Recue)==1)
	{
	//Si il s'agit d'un message automatique du Commodo Lumiere
	if(Trame_Recue.ident.extend.identificateur.ident==Ident_T_MesAuto_ComLum)
		{//il y donc eu forcement un changement d'tat
		Valeur_Commodo_Feux=~(Trame_Recue.data[1]); //attention pull-up=>invertion des bits
		// Affichage de l'tat commodo
		Flag.Affichage=1;
		Flag.Affichage_Com_Feux=1;
		//mettre a jour les nouveaux tat pour les feux:
		Valeur_FRG&=0x04;Valeur_FRD&=0x04;Valeur_FVG&=0x08;Valeur_FVD&=0x08; //RAZ de tous sauf les clignottant
		if(Cde_Phare) // Si Phare
			{Valeur_FVG=Cde_FV_P;Valeur_FVD=Cde_FV_P;  // Les feux aVant
 			 Valeur_FRG=Cde_FR_P;Valeur_FRD=Cde_FR_P;} // Les feux aRire
		else if(Cde_Code) // Si Code
			{Valeur_FVG=Cde_FV_C;Valeur_FVD=Cde_FV_C;  // Les feux aVant
 			 Valeur_FRG=Cde_FR_C;Valeur_FRD=Cde_FR_C;} // Les feux aRire
		else if(Cde_Veilleuse) // Si Veilleuse
			{Valeur_FVG=Cde_FV_V;Valeur_FVD=Cde_FV_V;  // Les feux aVant
 			 Valeur_FRG=Cde_FR_V;Valeur_FRD=Cde_FR_V;} // Les feux aRire
		/*Finalement l'activation des clignottant seront grs en externe*/ 
		if(Cde_Klaxon)
			{Valeur_FRG|=Masque_Klaxon;}
		if(Cde_Stop)
			{Valeur_FRG|=Masque_Stop;
			 Valeur_FRD|=Masque_Stop;}

		//Feux a actualiser
		 if(Valeur_FRG_Mem!=Valeur_FRG) //On compare la valeur mmoriser a la nouvelle
			{Flag.Env_Ordre_FRG=1;	  //afin d'autoriser l'envoie de la trame ssi changement
			 Valeur_FRG_Mem=Valeur_FRG;}//MAJ de la valeur memorise
		 if(Valeur_FRD_Mem!=Valeur_FRD)
			{Flag.Env_Ordre_FRD=1;
			 Valeur_FRD_Mem=Valeur_FRD;}
		 if(Valeur_FVG_Mem!=Valeur_FVG)
			{Flag.Env_Ordre_FVG=1;
			 Valeur_FVG_Mem=Valeur_FVG;}
		 if(Valeur_FVD_Mem!=Valeur_FVD)
			{Flag.Env_Ordre_FVD=1;
			 Valeur_FVD_Mem=Valeur_FVD;}

		//Mettre a jour les nouveau front de basculement du Commodo
		Flag.Env_Cfg_ComLum=1;
		}
	//Si il s'agit d'un message d'ACKitement du Commodo Lumiere
	if(Trame_Recue.ident.extend.identificateur.ident==Ident_T_AIM_Commodo_Feux)
		{
		if(Flag.ACK_Cfg_ComLum_Attendu==1) //on l'attendais suite a une config
			{
			gotoxy(2,1);
			printf("Commodo Lumiere correctement configure                   \n"); //on averti l'utilisateur
			Flag.ACK_Cfg_ComLum_Attendu=0;	//on RAZ le bit
			}
		}

	//Si il s'agit d'un message automatique du FRG
	if(Trame_Recue.ident.extend.identificateur.ident==Ident_T_MesAuto_FRG)
		{
		//on a recu un message donc il y a eu un changement d'tat sur les status!
		Valeur_Status_FRG=Trame_Recue.data[1]; //on recupere les status
		Flag.Affichage=1; //il faut actualiser l'tat de l'affichage
		Flag.Affichage_FRG=1;
		Flag.Env_Cfg_FRG=1; //Ainsi que la configuration des basculement d'tat des status
		}
	//Si il s'agit d'un message d'ACKitement du FRG
	if(Trame_Recue.ident.extend.identificateur.ident==Ident_T_AIM_FRG)
		{
		if(Flag.ACK_IM_FRG_Attendu==1) //on l'attendais suite a un ordre
			{
			gotoxy(2,2);
			printf("FRG, ordre correctement recu        \n"); //on averti l'utilisateur
			Flag.ACK_IM_FRG_Attendu=0;	//RAZ le bit
			Flag.Affichage=1; //il faut actualiser l'tat de l'affichage
			Flag.Affichage_FRG=1;
			}
		if(Flag.ACK_Cfg_FRG_Attendu==1) //on l'attendais suite a une configuration
			{
			gotoxy(40,2);
			printf("FRG, config correctement recu        \n"); //on averti l'utilisateur
			Flag.ACK_Cfg_FRG_Attendu=0;	//RAZ le bit
			}
		}

	//Si il s'agit d'un message automatique du FRD
	if(Trame_Recue.ident.extend.identificateur.ident==Ident_T_MesAuto_FRD)
		{
		//on a recu un message donc il y a eu un changement d'tat sur les status!
		Valeur_Status_FRD=Trame_Recue.data[1]; //on recupere les status
		Flag.Affichage=1; //il faut actualiser l'tat de l'affichage
		Flag.Affichage_FRD=1;
		Flag.Env_Cfg_FRD=1; //Ainsi que la configuration des basculement d'tat des status
		}

	//Si il s'agit d'un message d'ACKitement du FRD
	if(Trame_Recue.ident.extend.identificateur.ident==Ident_T_AIM_FRD)
		{
		if(Flag.ACK_IM_FRD_Attendu==1) //on l'attendais suite a un ordre
			{
			gotoxy(2,3);
			printf("FRD, ordre correctement recu        \n"); //on averti l'utilisateur
			Flag.ACK_IM_FRD_Attendu=0;	//RAZ le bit
			Flag.Affichage=1; //il faut actualiser l'tat de l'affichage
			Flag.Affichage_FRD=1;
			}
		if(Flag.ACK_Cfg_FRD_Attendu==1) //on l'attendais suite a une configuration
			{
			gotoxy(40,3);
			printf("FRD, config correctement recu        \n"); //on averti l'utilisateur
			Flag.ACK_Cfg_FRD_Attendu=0;	//RAZ le bit
			}
		}

	//Si il s'agit d'un message automatique du FVG
	if(Trame_Recue.ident.extend.identificateur.ident==Ident_T_MesAuto_FVG)
		{
		//on a recu un message donc il y a eu un changement d'tat sur les status!
		Valeur_Status_FVG=Trame_Recue.data[1]; //on recupere les status
		Flag.Affichage=1; //il faut actualiser l'tat de l'affichage
		Flag.Affichage_FVG=1;
		Flag.Env_Cfg_FVG=1; //Ainsi que la configuration des basculement d'tat des status
		}
	//Si il s'agit d'un message d'ACKitement du FVG
	if(Trame_Recue.ident.extend.identificateur.ident==Ident_T_AIM_FVG)
		{
		if(Flag.ACK_IM_FVG_Attendu==1) //on l'attendais suite a un ordre
			{
			gotoxy(2,4);
			printf("FVG, ordre correctement recu        \n"); //on averti l'utilisateur
			Flag.ACK_IM_FVG_Attendu=0;	//RAZ le bit
			Flag.Affichage=1; //il faut actualiser l'tat de l'affichage
			Flag.Affichage_FVG=1;
			}
		if(Flag.ACK_Cfg_FVG_Attendu==1) //on l'attendais suite a une configuration
			{
			gotoxy(40,4);
			printf("FVG, config correctement recu        \n"); //on averti l'utilisateur
			Flag.ACK_Cfg_FVG_Attendu=0;	//RAZ le bit
			}
		}

	//Si il s'agit d'un message automatique du FVD
	if(Trame_Recue.ident.extend.identificateur.ident==Ident_T_MesAuto_FVD)
		{
		//on a recu un message donc il y a eu un changement d'tat sur les status!
		Valeur_Status_FVD=Trame_Recue.data[1]; //on recupere les status
		Flag.Affichage=1; //il faut actualiser l'tat de l'affichage
		Flag.Affichage_FVD=1;
		Flag.Env_Cfg_FVD=1; //Ainsi que la configuration des basculement d'tat des status
		}
	//Si il s'agit d'un message d'ACKitement du FVD
	if(Trame_Recue.ident.extend.identificateur.ident==Ident_T_AIM_FVD)
		{
		if(Flag.ACK_IM_FVD_Attendu==1) //on l'attendais suite a un ordre
			{
			gotoxy(2,5);
			printf("FVD, ordre correctement recu        \n"); //on averti l'utilisateur
			Flag.ACK_IM_FVD_Attendu=0;	//RAZ le bit
			Flag.Affichage=1; //il faut actualiser l'tat de l'affichage
			Flag.Affichage_FVD=1;
			}
		if(Flag.ACK_Cfg_FVD_Attendu==1) //on l'attendais suite a une configuration
			{
			gotoxy(40,5);
			printf("FVD, config correctement recu        \n"); //on averti l'utilisateur
			Flag.ACK_Cfg_FVD_Attendu=0;	//RAZ le bit
			}
		}
	}//Fin des trames recue

/*Envoie des Trames de configurations********************************************************************************/
   if((Flag.Env_Cfg_FRG==1)||(Flag.Env_Cfg_FRD==1)||(Flag.Env_Cfg_FVG==1)||(Flag.Env_Cfg_FVD==1)||(Flag.Env_Cfg_ComLum==1))
      {
	if(Flag.Env_Cfg_ComLum==1)
		{
		//Mettre a jour les nouveau front de basculement du Commodo
	T_IM_Modules.ident.extend.identificateur.ident=Ident_T_IM_Commodo_Feux;
		T_IM_Modules.data[1]=0xFF;
		T_IM_Modules.data[2]=Valeur_Commodo_Feux;
		Ecrire_Trame(T_IM_Modules);	//envoie de la trame
		Flag.ACK_Cfg_ComLum_Attendu=1;
		gotoxy(2,1);
		printf("Commodo Lumiere, attente confirmation de configuration...\n"); //on averti l'utilisateur
		Flag.ACK_Cfg_ComLum_Attendu=1;
		Flag.Env_Cfg_ComLum=0;
		}
	if(Flag.Env_Cfg_FRG==1)
		{
		//Mettre a jour les nouveau front de basculement du FRG
		T_IM_Modules.ident.extend.identificateur.ident=Ident_T_IM_FRG;
		T_IM_Modules.data[1]=0xF0; //4 bit de poid fort sont concerns
		T_IM_Modules.data[2]=~(Valeur_Status_FRG); 
		Ecrire_Trame(T_IM_Modules);	//envoie de la trame
		Flag.ACK_Cfg_FRG_Attendu=1; //Le prochain ACKnolege sera donc un config
		gotoxy(40,2);
		printf("FRG, nouvelle config envoyee...     \n"); //on averti l'utilisateur
		Flag.Env_Cfg_FRG=0;
		}	
	if(Flag.Env_Cfg_FRD==1)
		{
		//Mettre a jour les nouveau front de basculement du FRG
		T_IM_Modules.ident.extend.identificateur.ident=Ident_T_IM_FRD;
		T_IM_Modules.data[1]=0xF0; //4 bit de poid fort sont concerns
		T_IM_Modules.data[2]=~(Valeur_Status_FRD); 
		Ecrire_Trame(T_IM_Modules);	//envoie de la trame
		Flag.ACK_Cfg_FRD_Attendu=1;
		gotoxy(40,3);
		printf("FRD, nouvelle config envoyee...     \n"); //on averti l'utilisateur
		Flag.Env_Cfg_FRD=0;
		}	
	if(Flag.Env_Cfg_FVG==1)
		{
		//Mettre a jour les nouveau front de basculement du FRG
		T_IM_Modules.ident.extend.identificateur.ident=Ident_T_IM_FVG;
		T_IM_Modules.data[1]=0xF0; //4 bit de poid fort sont concerns
		T_IM_Modules.data[2]=~(Valeur_Status_FVG); 
		Ecrire_Trame(T_IM_Modules);	//envoie de la trame
		Flag.ACK_Cfg_FVG_Attendu=1;
		gotoxy(40,4);
		printf("FVG, nouvelle config envoyee...     \n"); //on averti l'utilisateur
		Flag.Env_Cfg_FVG=0;
		}	
	if(Flag.Env_Cfg_FVD==1)
		{
		//Mettre a jour les nouveau front de basculement du FRG
		T_IM_Modules.ident.extend.identificateur.ident=Ident_T_IM_FVD;
		T_IM_Modules.data[1]=0xF0; //4 bit de poid fort sont concerns
		T_IM_Modules.data[2]=~(Valeur_Status_FVD); 
		Ecrire_Trame(T_IM_Modules);	//envoie de la trame
		Flag.ACK_Cfg_FVD_Attendu=1;
		gotoxy(40,5);
		printf("FVD, nouvelle config envoyee...     \n"); //on averti l'utilisateur
		Flag.Env_Cfg_FVD=0;
		}	
      }//Fin d'envoie de configuration aux modules

/*Envoie des Trames d'ordres aux Feux************************************************************************************/
   if((Flag.Env_Ordre_FRG==1)||(Flag.Env_Ordre_FRD==1)||(Flag.Env_Ordre_FVG==1)||(Flag.Env_Ordre_FVD==1))
      {
	if(Flag.Env_Ordre_FRG==1)
		{				
		//Preparer et envoyer les trame de pilotage des Feux
		T_IM_Feux.ident.extend.identificateur.ident=Ident_T_IM_FRG; //place l'identifiant adquate
		T_IM_Feux.data[2]=Valeur_FRG;	//place la valeur
		Ecrire_Trame(T_IM_Feux);	//envoie de la trame
		Flag.ACK_IM_FRG_Attendu=1;//On signale que le prochain ACKnolege sera celui consecutif a un pilotage (IM)
		gotoxy(2,2);	//on le signale a l'utilisateur
		printf("FRG, attente confirmation d'ordre...\n"); //on averti l'utilisateur
		Flag.Env_Ordre_FRG=0;//RAZ du BIT
		}

	if(Flag.Env_Ordre_FRD==1)
		{				
		T_IM_Feux.ident.extend.identificateur.ident=Ident_T_IM_FRD;
		T_IM_Feux.data[2]=Valeur_FRD;
		Ecrire_Trame(T_IM_Feux);
		Flag.ACK_IM_FRD_Attendu=1;
		gotoxy(2,3);	//on le signale a l'utilisateur
		printf("FRD, attente confirmation d'ordre...\n"); //on averti l'utilisateur
		Flag.Env_Ordre_FRD=0;//RAZ du BIT
		}

	if(Flag.Env_Ordre_FVG==1)
		{			
		T_IM_Feux.ident.extend.identificateur.ident=Ident_T_IM_FVG;
		T_IM_Feux.data[2]=Valeur_FVG;
		Ecrire_Trame(T_IM_Feux);
		Flag.ACK_IM_FVG_Attendu=1;
		gotoxy(2,4);	//on le signale a l'utilisateur
		printf("FVG, attente confirmation d'ordre...\n"); //on averti l'utilisateur
		Flag.Env_Ordre_FVG=0;//RAZ du BIT
		}

	if(Flag.Env_Ordre_FVD==1)
		{			
		T_IM_Feux.ident.extend.identificateur.ident=Ident_T_IM_FVD;
		T_IM_Feux.data[2]=Valeur_FVD;
		Ecrire_Trame(T_IM_Feux);
		Flag.ACK_IM_FVD_Attendu=1;
		gotoxy(2,5);	//on le signale a l'utilisateur
		printf("FVD, attente confirmation d'ordre...\n"); //on averti l'utilisateur
		Flag.Env_Ordre_FVD=0;//RAZ du BIT
		}
	//Flag.Affichage=1;//on est rentr=>on doit rafraichir l'afficage
      }//Fin envoie trame d'ordre aux feux

/*Gestion des glinottants***********************************************************************************************/
   if(Flag_Clignot!=2) 
	{//Si Flag Clignot!=2 
		//=>c.a.d que l'on a recu un message automatique du commodo, qui a actualiser Valeur_Commodo_Feux
		//=>donc l'IT c'est declanch est maintenant Flag Clignot est soit a un ou a 0
	if(Flag_Clignot==1) //1/2 periode d'allumage du clignottant
		{
		//on envoie qu'une seulle trame pour toute la dure,  et non une a chaque passage 
 		if(Cde_Warning==1) //Si Commande Warning 
			{//et si les tat feux ne correspondent pas
			 if(Clignot_FRG!=1)
				{Valeur_FRG|=Masque_Clign_AR; //Mise ne place du bit clignotant
				 Flag.Env_Ordre_FRG=1;} //Ordre d'envoie de la trame
			 if(Clignot_FRD!=1)
				{Valeur_FRD|=Masque_Clign_AR; //Mise ne place du bit clignotant
				 Flag.Env_Ordre_FRD=1;} //Ordre d'envoie de la trame
			 if(Clignot_FVG!=1)
				{Valeur_FVG|=Masque_Clign_AV; //Mise ne place du bit clignotant
				 Flag.Env_Ordre_FVG=1;} //Ordre d'envoie de la trame
			 if(Clignot_FVD!=1)
				{Valeur_FVD|=Masque_Clign_AV; //Mise ne place du bit clignotant
				 Flag.Env_Ordre_FVD=1;} //Ordre d'envoie de la trame
			 }
		else if(Cde_Clign_Droit)	// si clignottant Droit
			{
			 if(Clignot_FRD!=1)
				{Valeur_FRD|=Masque_Clign_AR; //Mise ne place du bit clignotant
				 Flag.Env_Ordre_FRD=1;} //Ordre d'envoie de la trame
			 if(Clignot_FVD!=1)
				{Valeur_FVD|=Masque_Clign_AV; //Mise ne place du bit clignotant
				 Flag.Env_Ordre_FVD=1;} //Ordre d'envoie de la trame
			}
 		else if(Cde_Clign_Gauche)	// si clignottant Gauche
			{
			 if(Clignot_FRG!=1)
				{Valeur_FRG|=Masque_Clign_AR; //Mise ne place du bit clignotant
				 Flag.Env_Ordre_FRG=1;} //Ordre d'envoie de la trame
			 if(Clignot_FVG!=1)
				{Valeur_FVG|=Masque_Clign_AV; //Mise ne place du bit clignotant
				 Flag.Env_Ordre_FVG=1;} //Ordre d'envoie de la trame
			 }
		}//Fin 1/2 periode d'allumage
	if(Flag_Clignot==0) //1/2 periode d'extinction du clignottant
		{
		//on envoie qu'une seulle trame pour toute la dure,  et non une a chaque passage 
 		if(Cde_Warning==1) //Si Commande Warning 
			{//et si les tat feux ne correspondent pas
			 if(Clignot_FRG!=0)
				{Valeur_FRG&=~Masque_Clign_AR; //Mise ne place du bit clignotant
				 Flag.Env_Ordre_FRG=1;} //Ordre d'envoie de la trame
			 if(Clignot_FRD!=0)
				{Valeur_FRD&=~Masque_Clign_AR; //Mise ne place du bit clignotant
				 Flag.Env_Ordre_FRD=1;} //Ordre d'envoie de la trame
			 if(Clignot_FVG!=0)
				{Valeur_FVG&=~Masque_Clign_AV; //Mise ne place du bit clignotant
				 Flag.Env_Ordre_FVG=1;} //Ordre d'envoie de la trame
			 if(Clignot_FVD!=0)
				{Valeur_FVD&=~Masque_Clign_AV; //Mise ne place du bit clignotant
				 Flag.Env_Ordre_FVD=1;} //Ordre d'envoie de la trame
			 }
		else if(Cde_Clign_Droit)	// si clignottant Droit
			{
			 if(Clignot_FRD!=0)
				{Valeur_FRD&=~Masque_Clign_AR; //Mise ne place du bit clignotant
				 Flag.Env_Ordre_FRD=1;} //Ordre d'envoie de la trame
			 if(Clignot_FVD!=0)
				{Valeur_FVD&=~Masque_Clign_AV; //Mise ne place du bit clignotant
				 Flag.Env_Ordre_FVD=1;} //Ordre d'envoie de la trame
			}
 		else if(Cde_Clign_Gauche)	// si clignottant Gauche
			{
			 if(Clignot_FRG!=0)
				{Valeur_FRG&=~Masque_Clign_AR; //Mise ne place du bit clignotant
				 Flag.Env_Ordre_FRG=1;} //Ordre d'envoie de la trame
			 if(Clignot_FVG!=0)
				{Valeur_FVG&=~Masque_Clign_AV; //Mise ne place du bit clignotant
				 Flag.Env_Ordre_FVG=1;} //Ordre d'envoie de la trame
			 }
		}//Fin 1/2periode d'extinction
	}//Fin du  if Flag_Clignot!=2 clignottant
	else 	{ //Si la commande clignottant n'est SUBITEMENT plus active, on verifie l'extinction des clignottant a l'aide des valeurs qui sont dans le status!!!
			 if((Image_Stat_FRD.bit.b2!=0))
				{Valeur_FRD&=~Masque_Clign_AR; //Mise ne place du bit clignotant
				 Image_Stat_FRD.bit.b2=0;//Anti bug Warning + Code / phare ou veilleuse lors d'un arret brutal
				 Flag.Env_Ordre_FRD=1;} //Ordre d'envoie de la trame
			 if((Image_Stat_FVD.bit.b3!=0))
				{Valeur_FVD&=~Masque_Clign_AV; //Mise ne place du bit clignotant
				 Image_Stat_FVD.bit.b3=0;//Anti bug 
				 Flag.Env_Ordre_FVD=1;} //Ordre d'envoie de la trame
			 if((Image_Stat_FRG.bit.b2!=0))
				{Valeur_FRG&=~Masque_Clign_AR; //Mise ne place du bit clignotant
				 Image_Stat_FRG.bit.b2=0;//Anti bug 
				 Flag.Env_Ordre_FRG=1;} //Ordre d'envoie de la trame
			 if((Image_Stat_FVG.bit.b3!=0))
				{Valeur_FVG&=~Masque_Clign_AV; //Mise ne place du bit clignotant
				 Image_Stat_FVG.bit.b3=0;//Anti bug 
				 Flag.Env_Ordre_FVG=1;} //Ordre d'envoie de la trame
	}//Fin du  if Flag_Clignot==2 clignottant

/*Gestion des l'affichage***********************************************************************************************/
   if(Flag.Affichage==1) 
	{//Il faut actualiser l'affichage
	if(Flag.Affichage_Com_Feux)
		{//du Commodo
		gotoxy(2,8);
		printf("Etat des differentes entrees imposees par le commodo:\n");
		printf(" Veilleuse=%d , Code=%d , Phare=%d , clign. G.=%d  	\n",Cde_Veilleuse,Cde_Code,Cde_Phare,Cde_Clign_Gauche);
		printf(" Klaxon=%d ,    Feux de stop=%d ,    clign. D= %d ,   Warning=%d	\n",Cde_Klaxon,Cde_Stop,Cde_Clign_Droit,Cde_Warning);
		Flag.Affichage_Com_Feux=0;
		}

	if(Flag.Affichage_FRG==1)
		{
		gotoxy(2,12);
		printf("Etat du Feux Arriere Gauche:\n");
		printf("Veuilleuse:");
		if(S_Veilleuse_FRG==0) printf("Probleme\n");
		else	{if(Veilleuse_FRG==1) printf("Allum OK\n");
			 else printf("Etteinte\n");}
		printf("Stop:");
		if(S_Stop_FRG==0) printf("Probleme\n");
		else	{if(Stop_FRG==1) printf("Allum OK\n");
			 else printf("Etteinte\n");}
		printf("Clignottant:");
		if(S_Clignot_FRG==0) printf("Probleme\n");
		else	{if(Clignot_FRG==1) printf("Allum OK\n");
			 else printf("Etteinte\n");}
		printf("Klaxon:");
		if(S_Klaxon_FRG==0) printf("Probleme\n");
		else	{if(Klaxon_FRG==1) printf("Allum OK\n");
			 else printf("Etteinte\n");}
		Flag.Affichage_FRG=0;
		}

	if(Flag.Affichage_FRD==1)
		{
		gotoxy(2,18);
		printf("Etat du Feux Arriere Droit:\n");
		printf("Veuilleuse:");
		if(S_Veilleuse_FRD==0) printf("Probleme\n");
		else	{if(Veilleuse_FRD==1) printf("Allum OK\n");
			 else printf("Etteinte\n");}
		printf("Stop:");
		if(S_Stop_FRD==0) printf("Probleme\n");
		else	{if(Stop_FRD==1) printf("Allum OK\n");
			 else printf("Etteinte\n");}
		printf("Clignottant:");
		if(S_Clignot_FRD==0) printf("Probleme\n");
		else	{if(Clignot_FRD==1) printf("Allum OK\n");
			 else printf("Etteinte\n");}
		printf("GP3:");
		if(S_Klaxon_FRD==0) printf("Probleme\n");
		else	{if(Klaxon_FRD==1) printf("Allum OK\n");
			 else printf("Etteinte\n");}
		Flag.Affichage_FRD=0;
		}

	if(Flag.Affichage_FVG==1)
		{
		gotoxy(2,24);
		printf("Etat du Feux Avant Gauche:\n");
		printf("Veuilleuse:");
		if(S_Veilleuse_FVG==0) printf("Probleme\n");
		else	{if(Veilleuse_FVG==1) printf("Allum OK\n");
			 else printf("Etteinte\n");}
		printf("Code:");
		if(S_Code_FVG==0) printf("Probleme\n");
		else	{if(Code_FVG==1) printf("Allum OK\n");
			 else printf("Etteinte\n");}
		printf("Phare:");
		if(S_Phare_FVG==0) printf("Probleme\n");
		else	{if(Phare_FVG==1) printf("Allum OK\n");
			 else printf("Etteinte\n");}
		printf("Clignottant:");
		if(S_Clignot_FVG==0) printf("Probleme\n");
		else	{if(Clignot_FVG==1) printf("Allum OK\n");
			 else printf("Etteinte\n");}
		Flag.Affichage_FVG=0;
		}

	if(Flag.Affichage_FVD==1)
		{
		gotoxy(2,30);
		printf("Etat du Feux Avant Droit:\n");
		printf("Veuilleuse:");
		if(S_Veilleuse_FVD==0) printf("Probleme\n");
		else	{if(Veilleuse_FVD==1) printf("Allum OK\n");
			 else printf("Etteinte\n");}
		printf("Code:");
		if(S_Code_FVD==0) printf("Probleme\n");
		else	{if(Code_FVD==1) printf("Allum OK\n");
			 else printf("Etteinte\n");}
		printf("Phare:");
		if(S_Phare_FVD==0) printf("Probleme\n");
		else	{if(Phare_FVD==1) printf("Allum OK\n");
			 else printf("Etteinte\n");}
		printf("Clignottant:");
		if(S_Clignot_FVD==0) printf("Probleme\n");
		else	{if(Clignot_FVD==1) printf("Allum OK\n");
			 else printf("Etteinte\n");}
		Flag.Affichage_FVD=0;
		}

	Flag.Affichage=0;
	}

	
/*detecter le q clavier pour quitter proprement*******************************************************************/
	if(SCSR&RDRF)
	{
	if((char)SCDR=='q') 
		{ Flag.Fin=1;}	//mise en place ud flag pour arreter le do ... while du prog principal
   	}//Fin detection appuie clavier

}while(Flag.Fin!=1); //tant que le q du clavier n'est pas appuy


/*Quitter proprement le programme**************************************************************************************/
   //On nous a donc demand de quitter, on restaure le systeme CAN:
   clsscr();
   //Inhiber l'interruption:   Void RestoreVect(int num)
   //SetVect(96,&irq_Stop); //installation vecteur
   //RestoreVect(96);
   //disable(96);

   //RAZ de la variable de fin de programme
   Flag.Fin=0;

   //Arret des message automatiques:
   T_IM_Modules.data[0]=0x1C;	//Acceder au registre IOINTEN
   T_IM_Modules.data[1]=0xFF;	//Prendre tout les bits en compte
   T_IM_Modules.data[2]=0x00;	//On veut inhiber les messages automatiques
   gotoxy(1,1);
   printf("Arret des messages automatique %c:\n",SCDR);
   gotoxy(2,2);
   printf("Feux Arriere Gauche:       \n");
   T_IM_Modules.ident.extend.identificateur.ident=Ident_T_IM_FRG;
   do{
   	gotoxy(2,3);
      }while(Cfg_Module(T_IM_Modules,Ident_T_AIM_FRG)!=1);
   gotoxy(2,4);
   printf("Feux Arriere Droit:       \n");
   T_IM_Modules.ident.extend.identificateur.ident=Ident_T_IM_FRD;
   do{
   	gotoxy(2,5);
      }while(Cfg_Module(T_IM_Modules,Ident_T_AIM_FRD)!=1);
   gotoxy(2,6);
   printf("Feux Avant Gauche:       \n");
   T_IM_Modules.ident.extend.identificateur.ident=Ident_T_IM_FVG;
   do{
   	gotoxy(2,7);
      }while(Cfg_Module(T_IM_Modules,Ident_T_AIM_FVG)!=1);
   gotoxy(2,8);
   printf("Feux Avant Droit:       \n");
   T_IM_Modules.ident.extend.identificateur.ident=Ident_T_IM_FVD;
   do{
   	gotoxy(2,9);
      }while(Cfg_Module(T_IM_Modules,Ident_T_AIM_FVD)!=1);
   gotoxy(2,10);
   printf("Commodo Feux:       \n");
   T_IM_Modules.ident.extend.identificateur.ident=Ident_T_IM_Commodo_Feux;
   do{
   	gotoxy(2,11);
      }while(Cfg_Module(T_IM_Modules,Ident_T_AIM_Commodo_Feux)!=1);

   //RAZ des Sorties
   T_IM_Modules.data[0]=0x1E;	//Acceder au registre GPLAT
   T_IM_Modules.data[1]=0x0F;	//Mask=> Seul les bits de poids faibles sont concernes
   T_IM_Modules.data[2]=0x00;	//Et on veut les etteindre
   gotoxy(1,13);
   printf("Extinction des Feux:\n");
   gotoxy(2,14);
   printf("Feux Arriere Gauche:       \n");
   T_IM_Modules.ident.extend.identificateur.ident=Ident_T_IM_FRG;
   do{
   	gotoxy(2,15);
      }while(Cfg_Module(T_IM_Modules,Ident_T_AIM_FRG)!=1);
   gotoxy(2,16);
   printf("Feux Arriere Droit:       \n");
   T_IM_Modules.ident.extend.identificateur.ident=Ident_T_IM_FRD;
   do{
   	gotoxy(2,17);
      }while(Cfg_Module(T_IM_Modules,Ident_T_AIM_FRD)!=1);
   gotoxy(2,18);
   printf("Feux Avant Gauche:       \n");
   T_IM_Modules.ident.extend.identificateur.ident=Ident_T_IM_FVG;
   do{
   	gotoxy(2,19);
      }while(Cfg_Module(T_IM_Modules,Ident_T_AIM_FVG)!=1);
   gotoxy(2,20);
   printf("Feux Avant Droit:       \n");
   T_IM_Modules.ident.extend.identificateur.ident=Ident_T_IM_FVD;
   do{
   	gotoxy(2,21);
      }while(Cfg_Module(T_IM_Modules,Ident_T_AIM_FVD)!=1);

   //Liberer le bus CAN:
   gotoxy(2,23);
   printf("Liberer le CAN\n");
   Stop_Aton_CAN();
   
   //Fin du programme!!
   gotoxy(2,27);
   printf("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n Fin du Programme\n!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n");

}//Fin du Programme principal !!!!


/*************************************************************************************************/
/****Sous programme de configuration pour les modules*******************/
/*************************************************************************************************/
char Cfg_Module(Trame T_Envoie,double ACK_Attendu)
{
char retour;		//valeur binaire du retour: 1 si OK, 0 si Erreur
int Cptr_TimeOut;		//Compteur pour attendre la reponse
Trame Trame_Recue;	//Trame de reception a analyser

Trame_Recue.ident.extend.identificateur.ident=0x00;	//mise a zero de l'identifiant de la trame de reception
Ecrire_Trame(T_Envoie);//Envoyer sur le reseau la trame demander

for(Cptr_TimeOut=0;Cptr_TimeOut<2000;Cptr_TimeOut++);	//Attendre une eventuelle reponse
Lire_Trame(&Trame_Recue);	//Recuperer la trame du bufferpour verifier l'identifiant

if(Trame_Recue.ident.extend.identificateur.ident==ACK_Attendu)	//c'est OK
	{
	printf("Le module a ete correctement configure\n");
	retour=1;
	}
else if(Trame_Recue.ident.extend.identificateur.ident==0x00)	//on a rien recu !!
	{
	printf("Pas de reponse du module.Verifier Alimentation!\n");	
	retour=0;
	}
else if(Trame_Recue.ident.extend.identificateur.ident==0x00)	//un autre module a repondu??
	{
	printf("Pas de reponse du module.On a recu un autre identifiant:%x!!!\n",Trame_Recue.ident.extend.identificateur.ident);	
	retour=0;
	}
return retour;
}

/*************************************************************************************************/
/****Sous programme d'interruption pour gerer le clignotant et la fin de programme*******************/
/*************************************************************************************************/
void irq_BDT()
{
compteur++;

//si clignotement dtecter comodo
if((Cde_Clign_Gauche==1)||(Cde_Clign_Droit==1)||(Cde_Warning==1)) 
	{
	if(compteur%Base_Temps_Clignot)		//si 1re moitier de la base de temps
		{	
		if(((compteur/Base_Temps_Clignot)&0x01)==0x01)
		Flag_Clignot=1; //allumer clignotant
		}
		else //si deuxime moitier de la base de temps
		{
		Flag_Clignot=0;//etteindre clignotant
		}
	}
	else Flag_Clignot=2; //pas d'ordre de clignottement




	//r-initialisation du compteur
	if(compteur>Base_Temp) compteur=0;
}