//=============================================================================================
//                   Bibliothque de fonctions pour la carte EID001
//---------------------------------------------------------------------------------------------
// Description : 
// Cette bibliothque permet de piloter la carte EID001 :
//    - Pilotage des LEDs
//    - Acquisition de l'etat des switchs
//    - Pilotage de l'afficheur 7 segments
//    - Acquisition de la valeur du potentiomtre PT2.
//
// Le port A est connect sur les LEDs , ce sera des sorties
// Le port B est connect sur les 8 switchs , ce sera des entres
// Le port C est connect sur l'afficheur 7 segments, ce sera des sorties
//---------------------------------------------------------------------------------------------
// Version du fichier : 1.0
// Date de cration : 9 Juin 2004
// Date de dernire modification : 9 Juin 2004
//=============================================================================================


// Fichiers a inclure
#include "eid210.h"
#include "eid001.h"

//=============================================================================================
//Conversion ascii=>hexa

char conv_ascii_hexa(char i)
{
int tmp;

for(tmp=0x30;tmp<0x3A;tmp++)
{
	if(i==tmp) return (i-0x30);
}

for(tmp=0x41;tmp<0x47;tmp++)
{
	if(i==tmp) return (i-0x37);
}

for(tmp=0x61;tmp<0x67;tmp++)
{
	if(i==tmp) return (i-0x57);
}
}

//=============================================================================================
// PARTIE 1 : GESTION DES LEDS

// Fonction : Initialisation du port A
void InitLED()
{
	CFSR2=0x8888;	// Configuration du port A en mode d'entrees/sorties simples,
	CFSR3=0x8888;	// soit en mode "DIO" (Discret Input Output)
	CPR1=0xFFFF;	// Configurer tous les bits en priorite haute
}


// Fonction : Pilotage des LEDs
void PilotLED(char Valeur)
{
	int Affiche = 0;
	char tmp = 0;
	
	tmp = Valeur & 0x80;
	if(tmp == (char) 0x80) Affiche |= 0x8000;
		else Affiche |= 0x4000; 

	tmp = Valeur & 0x40;
	if(tmp == 0x40) Affiche |= 0x2000;
		else Affiche |= 0x1000;

	tmp = Valeur & 0x20;
	if(tmp == 0x20) Affiche |= 0x0800;
		else Affiche |= 0x0400;

	tmp = Valeur & 0x10;
	if(tmp == 0x10) Affiche |= 0x0200;
		else Affiche |= 0x0100;

	tmp = Valeur & 0x08;
	if(tmp == 0x08) Affiche |= 0x0080;
		else Affiche |= 0x0040;

	tmp = Valeur & 0x04;
	if(tmp == 0x04) Affiche |= 0x0020;
		else Affiche |= 0x0010;

	tmp = Valeur & 0x02;
	if(tmp == 0x02) Affiche |= 0x0008;
		else Affiche |= 0x0004;

	tmp = Valeur & 0x01;
	if(tmp == 0x01) Affiche |= 0x0002;
		else Affiche |= 0x0001;

	HSRR1=Affiche;	// Chargement de la valeur dans le registre d'acces
				// aux canaux d'entrees/sorties
}
//=============================================================================================


//=============================================================================================
// PARTIE 2 : GESTION DES SWITCHS

// Fonction : Initialisation du port B
void InitSwitch()
{
	CFSR0=0x8888;	// Configuration du port A en mode d'entrees/sorties simples,
	CFSR1=0x8888;	// soit en mode "DIO" (Discret Input Output)
	CPR0=0xFFFF;	// Configurer tous les bits en priorite haute
	HSQR0=0xAAAA;	// Les entres seront mmorises dans les registre d'etat
				// lorsqu'on ecrira des "11" dans le registre de service (HSRR0)
}


// Fonction : Capture de la valeur des switchs
unsigned char CaptureSwitch()
{
	unsigned char Etat_Switch = 0;
	int i;

	HSRR0=0xFFFF;	//Demande d'acquisition des entrees

	for(i=0;i<1000;i++);	//Temporisation pour avoir le temps de recuperer l'etat des switchs

	// Recopie de l'image du Switch1 dans Etat_Switch
	if((Switch1 & 0x8000) == 0x8000) Etat_Switch |= 0x80;
	else Etat_Switch &= 0x7F;

	// Recopie de l'image du Switch2 dans Etat_Switch
	if((Switch2 & 0x8000) == 0x8000) Etat_Switch |= 0x40;
	else Etat_Switch &= 0xBF;

	// Recopie de l'image du Switch3 dans Etat_Switch
	if((Switch3 & 0x8000) == 0x8000) Etat_Switch |= 0x20;
	else Etat_Switch &= 0xDF;

	// Recopie de l'image du Switch4 dans Etat_Switch
	if((Switch4 & 0x8000) == 0x8000) Etat_Switch |= 0x10;
	else Etat_Switch &= 0xEF;

	// Recopie de l'image du Switch5 dans Etat_Switch
	if((Switch5 & 0x8000) == 0x8000) Etat_Switch |= 0x08;
	else Etat_Switch &= 0xF7;

	// Recopie de l'image du Switch6 dans Etat_Switch
	if((Switch6 & 0x8000) == 0x8000) Etat_Switch |= 0x04;
	else Etat_Switch &= 0xFB;

	// Recopie de l'image du Switch7 dans Etat_Switch
	if((Switch7 & 0x8000) == 0x8000) Etat_Switch |= 0x02;
	else Etat_Switch &= 0xFD;

	// Recopie de l'image du Switch8 dans Etat_Switch
	if((Switch8 & 0x8000) == 0x8000) Etat_Switch |= 0x01;
	else Etat_Switch &= 0xFE;
	
	return(Etat_Switch);	//On retourne l'image de l'etat des Switchs
}


// Fonction : Recopie de l'etat des switchs sur les LEDs
void RecopSwitchLED()
{
	PilotLED(CaptureSwitch());
}
//=============================================================================================


//=============================================================================================
// PARTIE 3 : GESTION DE L'AFFICHEUR 7 SEGMENTS

// Fonction : Initilisation de l'afficheur 7 segments
void Init7Seg()
{
	PortCDir = 0xFF;		// Tous les bits du port C configures en sortie
	PortCPort = AFF_T;	// On allume tous les segments
}


// Fonction : Pilotage de l'afficheur 7 segments
void Pilot7Seg(char Valeur)
{

	// Switch sur des nombres
	switch(Valeur)
	{
	case 0 :	// Affichage du caractere '0'
	case '0':
		PortCPort = AFF_0;
		break;

	case 1 :	// Affichage du caractere '1'
	case '1':
		PortCPort = AFF_1;
		break;

	case 2 :	// Affichage du caractere '2'
	case '2':
		PortCPort = AFF_2;
		break;

	case 3 :	// Affichage du caractere '3'
	case '3':
		PortCPort = AFF_3;
		break;

	case 4 :	// Affichage du caractere '4'
	case '4':
		PortCPort = AFF_4;
		break;

	case 5 :	// Affichage du caractere '5'
	case '5':
		PortCPort = AFF_5;
		break;

	case 6 :	// Affichage du caractere '6'
	case '6':
		PortCPort = AFF_6;
		break;

	case 7 :	// Affichage du caractere '7'
	case '7':
		PortCPort = AFF_7;
		break;

	case 8 :	// Affichage du caractere '8'
	case '8':
		PortCPort = AFF_8;
		break;

	case 9 :	// Affichage du caractere '9'
	case '9':
		PortCPort = AFF_9;
		break;

	case 0x0A :	// Affichage du caractere 'A'
	case 'a':
	case 'A':
		PortCPort = AFF_10;
		break;

	case 0x0B :	// Affichage du caractere 'B'
	case 'b':
	case 'B':
		PortCPort = AFF_11;
		break;

	case 0x0C :	// Affichage du caractere 'C'
	case 'c':
	case 'C':
		PortCPort = AFF_12;
		break;

	case 0x0D :	// Affichage du caractere 'D'
	case 'd':
	case 'D':
		PortCPort = AFF_13;
		break;

	case 0x0E :	// Affichage du caractere 'E'
	case 'e':
	case 'E':
		PortCPort = AFF_14;
		break;

	case 0x0F :	// Affichage du caractere 'F'
	case 'f':
	case 'F':
		PortCPort = AFF_15;
		break;

	case 0x10 :	// Affichage du caractere '.'
	case '.':
		PortCPort = AFF_P;
		break;

	case (char)0xFF :	// Tous les segments sont allumes
		PortCPort = AFF_T;
		break;
	}
}
//=============================================================================================


//=============================================================================================
// PARTIE 4 : GESTION DU POTENTIOMETRE 

// Fonction : Acquisition du resultat de la conversion analogique/numerique
unsigned short ResultConv()
{
	unsigned short result_conv = 0;

	CAN = 0x0040;		// Lancer la conversion sur la voie 0
	do
	{}while(IRQ_CAN == 1);	// On attend la fin de la conversion
	result_conv = CAN;	// On stocke le resultat de la conversion dans une variable
	return result_conv;	// On retourne le resultat
}


// Fonction : Affichage du resultat de la conversion sur les LEDs
void AffResultLED(unsigned short result_conv)
{
	unsigned char ValLED = 0;

	// On recupere le resultat de la conversion sur un octet
	ValLED = (unsigned char) ((result_conv & 0x0FFF) >> 4);
	PilotLED(ValLED);		// On affiche la valeur recupere sur les LEDs
}


// Fonction : Affichage du resultat de la conversion sur l'afficheur 7 segments
void AffResult7Seg(unsigned short result_conv)
{
	unsigned char Val7Seg = 0;
	int i;

	// On recupere les 4 bits de poids fort de la conversion
	Val7Seg = (unsigned char) ((result_conv & 0x0FFF) >> 8);
	Pilot7Seg(Val7Seg);	// On les affiche sur l'afficheur 7 segments
	Val7Seg = 0;
	for(i=0;i<1000000;i++);		// Tempo
	Pilot7Seg(0x10);
	for(i=0;i<1000000;i++);		// Tempo
	// On recupere les 4 bits de poids faible de la conversion
	Val7Seg = (unsigned char) ((result_conv & 0x00FF) >> 4);
	Pilot7Seg(Val7Seg);	// On les affiche sur l'afficheur 7 segments
}


// Fonction : Affichage du resultat de la conversion sur la sortie analogique 0 (LED D9)
void AffResultAna(unsigned short result_conv)
{
	unsigned char ValLED = 0;

	// On recupere le resultat de la conversion sur un octet
	ValLED = (unsigned char) ((result_conv & 0x0FFF) >> 4);
	SA0 = ValLED;
}
//=============================================================================================
