/*
 * Fichier d'entte pour mtr86-68K
 * Nota : certaines fonctions ne sont pas implmentes dans MTR68K
 *        voir la documentation du noyau
 * Auteur : D. Tschirhart & J.L. Rohou
 */
#include "bus_can.h"

#ifdef __cplusplus
extern "C" {
#endif

#define ushort unsigned short
#define uchar unsigned char
#define MTR_REQ

#define TACHE void
typedef void (*TACHE_ADR)(void);

#define MONITOR m_wait(); {
#define ENDM    m_signal(); }

#define SUSP    0X9000                      /* tat suspendu */
#define EXEC    0XA000                      /* tat xcution*/
#define PRET    0xC000                      /* tat ligible */
#define CREE    0x8000               /* tat cr ou dormant */
#define NCREE   0                           /* tat non cr */

#define  OK     0          /* pas d'erreur                   */
#define  NTE    1          /* numro de tche erron         */
#define  TIM    2          /* dpass. du dlai autorise     */
#define  PAE    3          /* pas de tche en attente d'vn.*/
#define  PRV    4          /* rendez-vous non initialis     */
#define  PDT    5          /* dpass. du nombre de tubes     */

#define  EOR    0xFFFFFFFF     /* opration OU sur vnements    */
#define  EAND   0x00000000     /* opration ET sur vnements    */

#define  ushort unsigned short
#define  uchar  unsigned char

#define SEMAPHORE unsigned
#define RESSOURCE unsigned
#define EVENEMENT unsigned
#define TUBE      unsigned
#define RV        unsigned
#define RVINIT    0x100

#define ETM	0
#define cprintf printf
#define cputs   printf
#define puts    printf
#define GotoXY  gotoxy
#define clrscr  clsscr
#define ClrScr   clsscr

void ClrScr(void);

#if 1
/*
 * Primive du noyau
 */

 int start_mtr(TACHE_ADR, int);

 void     MTR_REQ  mtr86exit  ( unsigned  n );
 void     MTR_REQ  mtr86fatal ( char *msg );
 unsigned MTR_REQ  dort       ( unsigned timeout );
 void     MTR_REQ  reveille   ( unsigned tache );
 unsigned MTR_REQ  cree       ( TACHE_ADR adr_tache, unsigned priorite,
                                unsigned size );
 unsigned MTR_REQ  tache_c    ( void );
 void     MTR_REQ  active     ( unsigned tache );
 unsigned MTR_REQ  suspend    ( unsigned tache );
 void     MTR_REQ  detruit    ( unsigned tache );
 void     MTR_REQ  supprime   ( unsigned tache );
 unsigned MTR_REQ  num_tache  ( TACHE_ADR adr_tache );
 void     MTR_REQ  m_send     ( unsigned tache, void *message );
 short    MTR_REQ  t_cree     ( TACHE_ADR adr_tache,unsigned size_stack,
				        unsigned tic);
 void     MTR_REQ  t_start    ( unsigned tache );
 void     MTR_REQ  t_stop     ( unsigned tache );
 unsigned MTR_REQ  mtr86status( unsigned tache );
 void     MTR_REQ  dsp_stk    ( void );
 void     MTR_REQ  m_wait     ( void );
 void     MTR_REQ  m_signal   ( void );
 unsigned MTR_REQ  com_open   ( unsigned com, ushort conf );
 void     MTR_REQ  com_close  ( unsigned com );
 unsigned MTR_REQ  com_write  ( unsigned com, unsigned c);
 unsigned MTR_REQ  com_read   ( unsigned com );
 unsigned MTR_REQ  mtr_vers   ( void );
 void     MTR_REQ  d_tic      ( ushort tic );
 unsigned MTR_REQ  cvrtic     ( long ms );
 void   * MTR_REQ  __tos      ( void );
 void     MTR_REQ  fct_disable( unsigned fonc );
 void     MTR_REQ  fct_enable ( unsigned fonc );

/*
 *  Gestionnaire de smaphores
 */

 SEMAPHORE MTR_REQ  s_cree   ( short valeur );                 /* Creer    */
 void      MTR_REQ  s_close  ( SEMAPHORE sem_num );            /* Dtruire */
 SEMAPHORE MTR_REQ  s_wait   ( SEMAPHORE sem_num, ushort timeout);/* P(s)  */
 void      MTR_REQ  s_init   ( SEMAPHORE sem_num, short valeur);/* init    */
 void      MTR_REQ  s_signal ( SEMAPHORE sem_num );            /* V(s)     */

/*
 *  Gestionnaire de ressources
 */

 RESSOURCE MTR_REQ  r_cree   ( void ) ;                        /* Creer    */
 void      MTR_REQ  r_close  ( RESSOURCE n_res );              /* Dtruire */
 RESSOURCE MTR_REQ  r_wait   ( RESSOURCE n_res );              /* P(s)     */
 void      MTR_REQ  r_signal ( RESSOURCE n_res );              /* V(s)     */

/*
 * Gestionnaire de tubes
 */

 TUBE      MTR_REQ  p_open   ( unsigned tache_w, unsigned tache_r,
			       unsigned timeout );
 void      MTR_REQ  p_close  ( TUBE tube );
 unsigned  MTR_REQ  p_write  ( TUBE tube, void *buf, unsigned size );
 unsigned  MTR_REQ  p_read   ( TUBE tube, void *buf, unsigned size );
 unsigned  MTR_REQ  p_plein  ( TUBE n );
 unsigned  MTR_REQ  p_vide   ( TUBE n );

/*
 * Gestionnaire d'vnements
 */

 EVENEMENT MTR_REQ  e_cree   ( void );
 void      MTR_REQ  e_close  ( EVENEMENT evn );
 unsigned  MTR_REQ  e_wait   ( EVENEMENT evn, ushort valeur,ushort mask,\
			             ushort type_op, ushort timeout );
 unsigned  MTR_REQ  e_signal ( EVENEMENT evn, ushort valeur );

 /*
  * Gestionnaire d'it
  */

 short     MTR_REQ  isr_cree ( TACHE_ADR adr_tache, unsigned vecteur );
 void      MTR_REQ  isr_close( unsigned tache );
 void      MTR_REQ  isr_in   ( unsigned tache );
 void      MTR_REQ  isr_end  ( unsigned tache );

 /*
  * Gestionnaire de rendez-vous
  */

 RV        MTR_REQ rv_cree   ( unsigned n, unsigned timeout );
 void      MTR_REQ rv_close  ( RV rvn );
 unsigned  MTR_REQ rv_signal ( RV rvn );

 /*
  * Gestionnaire des lignes de communication
  */

/*
 * Structures de donnes pour les ports de communication OEM
 */

 /* Canaux PC standards : COM 1..4*/

 unsigned MTR_REQ  com_open   ( unsigned com, ushort conf );
 void     MTR_REQ  com_close  ( unsigned com );
 unsigned MTR_REQ  com_write  ( unsigned com, unsigned c );
 unsigned MTR_REQ  com_read   ( unsigned com );

 /* Gestionnaire du bus CAN  travers SJA1000 carte ATON sur buc PC104 */

	void MTR_REQ OpenCAN(int adr);
	void MTR_REQ CloseCAN();
	void MTR_REQ EcrireTrame(Trame *p_trame);
	int MTR_REQ LireTrame(Trame *p_trame);
	void MTR_REQ AfficheTrame(Trame *p_trame);


#endif

#ifdef __cplusplus
}
#endif