	/***********************************************************************************************
	*       TP sur  EID210 / Rseau CAN - VMD  (Vhicule Multiplex Didactique)					
	************************************************************************************************
	*  	TP n 3: Vrifier le fonctionnement d'un bloc optique
	*----------------------------------------------------------------------------------------------			
	*   CAHIER DES CHARGES :					      						
	*  *********************						      						
	* On souhaite en enchanement cyclique des diffrents tats du bloc optique avant droit
	*	(Rien, Veilleuse, Stop, Phare ,clignottant etc).
	*  On ralisera en plus des fonctions de controle:								
	*	- On vrifiera que le module command renvoi bien une trame d'acquittement.
	*	- On vrifiera (par fonction logicielle) que les lampes commandes sont effectivement 
	*		allumes 
	*	- On affichera quelles sont les lampes concernes ainsi que le rsultat du test.
      *    	- priode de permutation des lampes 3,6 S,
 	*----------------------------------------------------------------------------------------------	

	*************************************************************************************************/
// Fichiers  inclure
//********************

#include <stdio.h>
#include "Structures_donnees.h"
#include "mtr86.h"
#include "vmd.h"

TACHE  init  (void);
TACHE  LectureTrame(void);
TACHE	 chenillard(void);
TACHE  EcrireFeux();
TACHE  Verif_Etat_Feux(void);
TACHE  Affichage();

/************************************************************************/
/*				Tache d'initialisation					*/
/************************************************************************/
TACHE init(void)
{
OpenCAN(ATON_V2);	//ATON_V1 pour les anciennes cartes ATONCAN
			//ATON_V2 pour les nouvelles cartes ATONCAN
active(cree(chenillard,1,1024));	//en permanance active
cree(EcrireFeux,2,1024);		//Cre en priorit 2 pour raliser une criture, avant
cree(Verif_Etat_Feux,3,1024);		// de verifier l'etat rel du bloc optique
active(cree(LectureTrame,4,1024));	//Reception des trames
cree(Affichage,5,1024);			//L'affichage conserve la derniere des priorite
}

/************************************************************************/
/*				Tache gerant le chenillard				*/
/************************************************************************/
TACHE chenillard(void)
{
static p_s=0x00;
while(1)
	{
	switch(p_s)
			{
			case 0 : p_s=0x01;
				 break;
			case 1 : p_s=0x02;
				break;
			case 2 : p_s=0x04;
				break;
			case 4 : p_s=0x08;
				break;
			case 8 : p_s=0;
				break;
		}	
	m_send(num_tache(EcrireFeux),&p_s);	//Activation de la tache d'ecriture
	active(num_tache(Verif_Etat_Feux)); //Activation de la tache de verification
	dort(360); //dort 3,6 secondes
	}
}

/***********************************************************************/
/*		Tache gerant l'envoie des trames sur le Rseau CAN		*/
/*    pour allumer le bloc optique							*/
/***********************************************************************/
TACHE  EcrireFeux(e_s)
char e_s;
{
//initialisation de  la trame de commande sur le Feux aVant Gauche
Trame T_IM_Feux;
T_IM_Feux.trame_info.registre=0x00;
T_IM_Feux.trame_info.champ.extend=1;
T_IM_Feux.trame_info.champ.dlc=0x03;
T_IM_Feux.ident.extend.identificateur.ident=Ident_T_IM_FVG;
T_IM_Feux.data[0]=0x1e;	
T_IM_Feux.data[1]=0x0f;
//Envoie sur le Bus Can la trame comprennant le feux  activer
T_IM_Feux.data[2]=e_s;
EcrireTrame(&T_IM_Feux);
//endormissement infinie de la tache (activation sur reveille ou m_send)
}

/************************************************************************/
/*		Tache gerant l'envoie des trames sur le Rseau CAN		*/
/*    pour verifier l'etat du bloc optique					*/
/************************************************************************/
TACHE  Verif_Etat_Feux(void)
{
//initialisation de  la trame de verification d'etat du Feux aVant Gauche
Trame T_IRM_Feux;
T_IRM_Feux.trame_info.registre=0x00;
T_IRM_Feux.trame_info.champ.extend=1;
T_IRM_Feux.trame_info.champ.dlc=0x01;
T_IRM_Feux.trame_info.champ.rtr=1;
T_IRM_Feux.ident.extend.identificateur.ident=Ident_T_IM_FVG;
//Envoie sur le Bus Can la trame interrogative
EcrireTrame(&T_IRM_Feux);
//endormissement infinie de la tache (activation sur reveille ou m_send)
}

/************************************************************************/
/*		Reception et Gestion des trames recues				*/
/************************************************************************/
TACHE LectureTrame(void)
{
static Trame tr_rx;
static Valeur_FVG_Mem;
while(1)
	{
	LireTrame(&tr_rx);

	if (tr_rx.ident.extend.identificateur.ident==Ident_T_AIM_FVG)
		{
		m_send(num_tache(Affichage),&tr_rx);
		}
	if (tr_rx.ident.extend.identificateur.ident==Ident_T_IRM_FVG)
		{
		Valeur_Status_FVG=tr_rx.data[0];	//Recupere l'etat du status qui est dans le poid fort de data[0]
		Valeur_FVG=tr_rx.data[0];		//Recupere l'etat demand qui est dans le poid faible de data[0]
		if(Valeur_FVG_Mem!=Valeur_FVG)
			{
			Valeur_FVG_Mem=Valeur_FVG;
			m_send(num_tache(Affichage),&tr_rx);
			}
		}
	}
}

/***********************************************************************/
/*		Affichage des trames et de l'tat rel des lampes		*/
/***********************************************************************/
TACHE Affichage(tram)
Trame tram;
{
MONITOR
if(tram.ident.extend.identificateur.ident==Ident_T_AIM_FVG)
	gotoxy(1,7);
if(tram.ident.extend.identificateur.ident==Ident_T_IRM_FVG)
	{
	gotoxy(1,11);
	if(Veilleuse_FVG)	//on a demander d'allumer la veilleuse
				//on verifie que le status soit conforme  la demande
		if(S_Veilleuse_FVG) printf("Veilleuse OK \n");	//Status est  1, c'est bon
			else printf("Veilleuse HS \n");		//Status est  0, l'etat n'est pas conforme  la demmande
	else if(Code_FVG)	//on a demander d'allumer le code
				//on verifie que le status soit conforme  la demande
		if(S_Code_FVG) printf("Code OK      \n");
			else printf("Code HS      \n");
	else if(Phare_FVG)	//on a demander d'allumer le phare
				//on verifie que le status soit conforme  la demande
		if(S_Phare_FVG) printf("Phare OK     \n");
			else printf("Phare HS     \n");
	else if(Clignot_FVG)	//on a demander d'allumer le clignotant
		if(S_Clignot_FVG) printf("Clignotant OK\n");
			else printf("Clignotant HS\n");
	gotoxy(1,9);
	}
AfficheTrame(&tram);
ENDM
}

/************************************************************************/
/*				Programme principal					*/
/************************************************************************/
main()
{
clsscr();
printf("tp3_bus can avec mtr86\n");
gotoxy(1,4);
printf("Trames Recues\n");
gotoxy(1,6);
printf("Acknolege\n");
gotoxy(1,8);
printf("Etat du feux\n");
start_mtr(init,512);
}