	/*******************************************************************************************
	*       TP sur  EID210 / Rseau CAN - VMD  (Vhicule Multiplex Didactique)			
	********************************************************************************************
	*   TP n7_2:  Configuration des modules pour un envoie de trame lors d'un changement d'tat
	*-------------------------------------------------------------------------------------------	
	*   CAHIER DES CHARGES :					      				
	*  *********************						      			
	*  On souhaite une emmission spontane de la part des commodos Feux et EG, ainsi que 
	*	de la carte asservissement.
	*  L'envoie se fera lors d'une modification d'etat sur une des entres
	*  On ralisera en plus des fonctions suivante:								
	*	- Receotion des trames.
	*	- Affichage de l'tat du Commodo Feux, du Commodo Essuie Glace, et des fins de course
	*------------------------------------------------------------------------------------------

	*******************************************************************************************/
// Dclaration des fichiers d'inclusion
#include <stdio.h>
#include "Structures_donnees.h"
#include "mtr86.h"
#include "vmd.h"

//Prototype des taches
TACHE  init  (void);
TACHE  LectureTrame(void);
TACHE  init_commodo(void);
TACHE Affichage(void);

/******************************************************************************************************************/
/*					Tache d'initialisation du VMD (configuration des modules)					*/
/******************************************************************************************************************/
TACHE init(void)
{
Trame t_gpdr,cfg_envoie_auto,T_IM_CFG,Trame_Recue;

//placer les datas
int cpt;
//Tableau comprennant toutes les donnes.
//Les 2 premieres valeurs sont pour le Convertisseur Analogique Numerique de la molette du commodo EG.
//Les suivantes sont pour l'Asservissemnt (cfg PWM, frequence, Validation puissance,...)
int datazero[]={0x2A,0x2B,0x21,0x23,0x22,0x24,0x25,0x26,0x1E};
int dataun[]={0xF0,0xFF,0xB3,0xFF,0xB3,0xFF,0xFF,0xFF,0x10};
int datadeux[]={0x80,0x0E,0x80,0xFF,0x80,0xFF,0x00,0x00,0x10};

OpenCAN(ATON_V2);	//ATON_V1 pour les anciennes cartes ATONCAN
			//ATON_V2 pour les nouvelles cartes ATONCAN

//configuration des Entres/Sorties
t_gpdr.trame_info.registre=0x00;
t_gpdr.trame_info.champ.extend=1; // On travaille en mode tendu
t_gpdr.trame_info.champ.dlc=0x03; // Il y aura 3 donnes de 8 bits (3 octets envoys)
//Commodo Feux
t_gpdr.ident.extend.identificateur.ident=Ident_T_IM_Commodo_Feux;
t_gpdr.data[0]=0x1F;
t_gpdr.data[1]=0x7F; 
t_gpdr.data[2]=0x7F; 
EcrireTrame(&t_gpdr);
while((LireTrame(&Trame_Recue)==0));
//Comoodo EG
t_gpdr.ident.extend.identificateur.ident=Ident_T_IM_Commodo_EG;
t_gpdr.data[2]=0x7F; // troisime donne -> "Valeur" -> Tous les bits en entre
EcrireTrame(&t_gpdr);
while((LireTrame(&Trame_Recue)==0));
//Carte Asservissement
t_gpdr.ident.extend.identificateur.ident=Ident_T_IM_Asservissement; 
t_gpdr.data[2]=0xE3;
EcrireTrame(&t_gpdr);
while((LireTrame(&Trame_Recue)==0));

//configuration du convertiseur Ana->Num du commodo EG
T_IM_CFG.trame_info.registre=0x00;
T_IM_CFG.trame_info.champ.extend=1; // On travaille en mode tendu
T_IM_CFG.trame_info.champ.dlc=0x03; // Il y aura 3 donnes de 8 bits (3 octets envoys)
T_IM_CFG.ident.extend.identificateur.ident=Ident_T_IM_Commodo_EG; 
for(cpt=0;cpt<2;cpt++)
{
T_IM_CFG.data[0]=datazero[cpt]; 	// Adresse du registre
T_IM_CFG.data[1]=dataun[cpt]; 	// Masque -> bits concerns
T_IM_CFG.data[2]=datadeux[cpt];	// Valeur 
EcrireTrame(&T_IM_CFG);
while((LireTrame(&Trame_Recue)==0));
}

//Configuration specifique du module Asservissement
T_IM_CFG.trame_info.registre=0x00;
T_IM_CFG.trame_info.champ.extend=1; // On travaille en mode tendu
T_IM_CFG.trame_info.champ.dlc=0x03; // Il y aura 3 donnes de 8 bits (3 octets envoys)
T_IM_CFG.ident.extend.identificateur.ident=Ident_T_IM_Asservissement; 
for(cpt=0;cpt<9;cpt++)
{
T_IM_CFG.data[0]=datazero[cpt]; 	// Adresse du registre
T_IM_CFG.data[1]=dataun[cpt]; 	// Masque -> bits concerns
T_IM_CFG.data[2]=datadeux[cpt];	// Valeur 
EcrireTrame(&T_IM_CFG);
while((LireTrame(&Trame_Recue)==0));
}

//Configuration du composant pour un envoie automatique des trames ...
cfg_envoie_auto.trame_info.registre=0x00;
cfg_envoie_auto.trame_info.champ.extend=1; 
cfg_envoie_auto.trame_info.champ.dlc=0x03; 
//Commodo Feux
cfg_envoie_auto.ident.extend.identificateur.ident=Ident_T_IM_Commodo_Feux;
cfg_envoie_auto.data[0]=0x1C;	//Adresse du registre IOTEN=0x00 + Offset=0x1C==>0x1C 
cfg_envoie_auto.data[1]=0xFF; //Tous les bits sont concerns 
cfg_envoie_auto.data[2]=0xFF; //GP7TXC  GP0TXC sont valids pour un envoie sur changement d'tat
EcrireTrame(&cfg_envoie_auto);
while((LireTrame(&Trame_Recue)==0));

//Commodo EG
cfg_envoie_auto.ident.extend.identificateur.ident=Ident_T_IM_Commodo_EG;
EcrireTrame(&cfg_envoie_auto);
while((LireTrame(&Trame_Recue)==0));
//Asservissement
cfg_envoie_auto.ident.extend.identificateur.ident=Ident_T_IM_Asservissement;
EcrireTrame(&cfg_envoie_auto);
while((LireTrame(&Trame_Recue)==0));

MONITOR
clsscr();
//L'affichage prend ennormement de temps, donc on anticipe
gotoxy(1,1);
printf("**************************tp4_bus can avec mtr86**************************\n");
printf(" Etat du Commodo Essuie Glace                                             \n");
printf("  GP0: Cde_EG_Av_Int=          							    \n");
printf("  GP3: Cde_EG_Av_Pos1=        , GP4: Cde_EG_Av_Pos2=                      \n");
printf("  GP7: Cde_Lave_Glace_Av=   							    \n");
printf("  GP1: Entree1=               , GP2: Entree2=       			    \n");
printf("  GP5: Cde_EG_Ar=             , GP6: Cde_Lave_Glace_Ar=   		    \n");
printf("  				        							    \n");
printf(" Etat du Commodo Feux                                                     \n");
printf("  GP0: Veilleuse=      ,GP2: Phare=        ,GP3: Code=                    \n");
printf("  GP1: Warning=        ,GP4: ClinGauche=   ,GP5: ClinDroit=               \n");
printf("  GP6: Feux stop=      ,GP7: Klaxon=                                      \n");
printf("  				        							    \n");
printf(" Etat des fins de course sur la carte Asservissemnt			    	    \n");
printf("  Fin de Course : Gauche=       	  Sur-course=          Droit=		    \n");
ENDM

active(cree(LectureTrame,1,1024));
cree(Affichage,2,1024);
}

/******************************************************************************************************************/
/*						Lecture des trames recue									*/
/******************************************************************************************************************/
TACHE LectureTrame(void)
{
static Trame tr_rx,Cfg_IointP0;
//Cette trame nous permet de definir la transition sur un front montant ou descendant
Cfg_IointP0.trame_info.registre=0x00;
Cfg_IointP0.trame_info.champ.extend=1; 
Cfg_IointP0.trame_info.champ.dlc=0x03; 
Cfg_IointP0.data[0]=0x1D;	//Adresse du registre IOINTP0=0x01 + Offset=0x1C==>0x1D 
Cfg_IointP0.data[1]=0xFF; //Tous les bits sont concerns 

while(1)
	{
	if(LireTrame(&tr_rx))
		{
		MONITOR
		gotoxy(1,16);
		printf("Trame_recue\n");
		if (tr_rx.ident.extend.identificateur.ident==0x05400000) 
			{
			Valeur_Commodo_Feux=~tr_rx.data[1];
			gotoxy(1,17);
			printf("Reponse du Commodo Feux,%x  \n",Valeur_Commodo_Feux);
			//la transition est modifier en fonction de l'tat des entres (si on etait sur detection
			// d'un front montant, on passe en detection d'un front descendant et inversement)
			Cfg_IointP0.ident.extend.identificateur.ident=Ident_T_IM_Commodo_Feux;
			Cfg_IointP0.data[2]=Valeur_Commodo_Feux; 
			EcrireTrame(&Cfg_IointP0);
			}
		if (tr_rx.ident.extend.identificateur.ident==0x05C00000) 
			{
			Valeur_Commodo_EG=~tr_rx.data[1];
			gotoxy(1,18);
			printf("Reponse du Commodo EG,%x  \n",Valeur_Commodo_EG);
			//la transition est modifier en fonction de l'tat des entres (si on etait sur detection
			// d'un front montant, on passe en detection d'un front descendant et inversement)
			Cfg_IointP0.ident.extend.identificateur.ident=Ident_T_IM_Commodo_EG;
			Cfg_IointP0.data[2]=Valeur_Commodo_EG; 
			EcrireTrame(&Cfg_IointP0);
			}
		if (tr_rx.ident.extend.identificateur.ident==0x00C00000) 
			{
			Valeur_FC_EG=~tr_rx.data[1];
			gotoxy(1,19);
			printf("Reponse de l'asv,%x  \n",Valeur_FC_EG);
			//la transition est modifier en fonction de l'tat des entres (si on etait sur detection
			// d'un front montant, on passe en detection d'un front descendant et inversement)
			Cfg_IointP0.ident.extend.identificateur.ident=Ident_T_IM_Asservissement;
			Cfg_IointP0.data[2]=Valeur_FC_EG; 
			EcrireTrame(&Cfg_IointP0);
			}
		gotoxy(1,20);
		AfficheTrame(&tr_rx);
		active(num_tache(Affichage));
		ENDM
		dort(20);
		}
	}
}

/******************************************************************************************************************/
/*			Tache d'affichage	(mise a jour de l'etat du systeme sur le moniteur)					*/
/******************************************************************************************************************/
TACHE Affichage(void)
{
static Valeur_Commodo_Feux_Aff;
static Valeur_Commodo_EG_Aff;	
static Valeur_FC_EG_Aff;

if(Valeur_Commodo_EG_Aff!=Valeur_Commodo_EG)
//on actualise les valeurs du commodo EG sur le Moniteur
	{	
	Valeur_Commodo_EG_Aff=Valeur_Commodo_EG;
	MONITOR		
	gotoxy(22,3);
	printf("%d\n",Cde_EG_Av_Int);
	gotoxy(16,6);
	printf("%d\n",Entree1);
	gotoxy(46,6);
	printf("%d\n",Entree2);
	gotoxy(23,4);
	printf("%d\n",Cde_EG_Av_Pos1);
	gotoxy(53,4);
	printf("%d\n",Cde_EG_Av_Pos2);
	gotoxy(18,7);
	printf("%d\n",Cde_EG_Ar);
	gotoxy(56,7);
	printf("%d\n",(Cde_Lave_Glace_Ar^0x01));
	gotoxy(27,5);
	printf("%d\n",(Cde_Lave_Glace_Av^0x01));
	ENDM
	}

if(Valeur_FC_EG_Aff!=Valeur_FC_EG)
//on actualise les valeurs des fin de course sur le Moniteur
	{
	Valeur_FC_EG_Aff=Valeur_FC_EG;
	MONITOR
	gotoxy(26,15);
	if(fcg)	//Fin de course gauche actif
		printf("1\n");
	else
		printf("0\n");

	gotoxy(62,15);
	if(fcd)	//Fin de course droit actif
		printf("1\n");
	else
		printf("0\n");

	gotoxy(46,15);
	if(fs)	//Fin de surcouse
		printf("1\n");
	else
		printf("0\n");
	ENDM
	}
if(Valeur_Commodo_Feux_Aff!=Valeur_Commodo_Feux)
//on actualise les valeurs du commodo Feux sur le Moniteur
	{
	Valeur_Commodo_Feux_Aff=Valeur_Commodo_Feux;
	MONITOR
	gotoxy(18,10);
	printf("%d\n",Cde_Veilleuse);
	gotoxy(36,10);
	printf("%d\n",Cde_Phare);
	gotoxy(55,10);
	printf("%d\n",Cde_Code);
	gotoxy(16,11);
	printf("%d\n",Cde_Warning);
	gotoxy(41,11);
	printf("%d\n",Cde_Clign_Gauche);
	gotoxy(60,11);
	printf("%d\n",Cde_Clign_Droit);
	gotoxy(18,12);
	printf("%d\n",Cde_Stop);
	gotoxy(37,12);
	printf("%d\n",Cde_Klaxon);
	ENDM
	}
}

/******************************************************************************************************************/
/*						Programme principal										*/
/******************************************************************************************************************/
main()
{
clsscr();
printf("tp1_bus can avec mtr86\n");
start_mtr(init,1200);
}