	/*********************************************************************
	*     TP sur EID210 et EID060  							
	**********************************************************************
	*  Cahier des charges:					      	
	*  *******************					      	
	*   Acquisition de la rponse temporelle d'une regulation en POSITION
	*	suite a un chelon de consigne
	*	-> Le correcteur sera de type "Proportionnel"								
	*	-> La valeur de l'echelon est choisi par le potentiometre
	*	-> Les chantillons acquis sont transmis par RS232	
	* 	-> On trace la courbe de rponse sous EXCEL		 
	* -------------------------------------------------------------------
	*  NOM du FICHIER:  TP5_12_BF_Pos_P.C	    		
	* ****************							
	*********************************************************************/
/* Liste des fichiers  inclure */
#include "Cpu_reg.h"			// Registre du processeur 68332
#include "EID210_reg.h"			// 
#include "Structures_donnees.h"
#include "EID060.h"


#define S_Mesure_Position  CPTR_Pos_Mot  // Rsultat de comptage des signaux en quadrature de phase
#define Cde_Moteur  SA0 //La commande moteur se fait par la Sortie Analogique 0 de la carte processeur EID210 
#define Valeur_Te  5   // La priode d'chantillonnage de mesure vitesse sera de 5 mS
#define Valeur_Tea  10   // La priode d'chantillonnage pour l'acquisition sera de 10 mS
#define NB_Max_Echantillons  190
#define BP_CRTL REG_ETAT.bits_octet.b8   // Bouton poussoir repere "CRTL sur carte EID210
// Pour Correcteur
#define K1   5   // Ce qui donne un coefficient d'action proportionnelle 10/16 = 0,625

/*  Declaration des variables
==============================*/


// Pour periode d'echantillonnage mesure vitesse
unsigned char Cptr_Te;
// Pour periode d'echantillonnage acquisition
unsigned char Cptr_Tea;
// Pour la mesure de la vitesse
short  S_Mesure_Vitesse;
short S_Mesure_Position_Precedente;
// Pour le regulateur numrique
int I_Resultat_Calcul,I_Resultat_Calcul_1,I_Ecart,I_Ecart_1;
short S_Consigne;
short S_Valeur_Pot;
// Pour la transmission des echantillons
char TC_Caracteres_A_Emettre[8]; // Tableau des caractere a emettre pour l'echantillon acquis
char C_Rang_Caractere_A_Emettre; // Rang du caractere a emettre
unsigned short C_Compteur_Echantillons;
short S_Echantillon_Acquis;
//  Pour affichage
unsigned short US_Compteur_Affichage=0;
// Pour attente
unsigned int UI_Compteur_Attente;
// Indicateurs Binaires
union byte_bits Indicateurs;
#define IB_Autorisation_Acquisition  Indicateurs.bit.b0
#define IB_Echantillon_Acquis Indicateurs.bit.b1
#define IB_Emission_En_Cours Indicateurs.bit.b2

/* FONCTION d'interruption pour temporisation  (base de temps: toutes les mS)
===============================================================================*/
void Irq_Base_De_Temps(void)
{   Cptr_Te--;
	if (Cptr_Te==0)
	{Cptr_Te=Valeur_Te;
	
	//Algorithme de regulation
	I_Ecart=S_Consigne-S_Mesure_Position;  // Calcul de l'ecart
	I_Resultat_Calcul=+ K1*I_Ecart;
	if(I_Resultat_Calcul>0)I_Resultat_Calcul+=200;  // Compensation du seuil positif
	if(I_Resultat_Calcul<0)I_Resultat_Calcul-=200;  // Compensation du seuil ngatif
	I_Resultat_Calcul=I_Resultat_Calcul>>4; 
	if(I_Resultat_Calcul>100)I_Resultat_Calcul=100;  // On limite  la plage de proportionnalit
	if(I_Resultat_Calcul<-100)I_Resultat_Calcul=-100;// voir caractristiaue en BO
	I_Resultat_Calcul+=135; // Valeur centre de symtrie caractristique en BO 
	Cde_Moteur=(unsigned char)(I_Resultat_Calcul); // application  la commande du moteur
	}
  if(IB_Autorisation_Acquisition==1)
	{Cptr_Tea--;
	 if (Cptr_Tea==0)
		{Cptr_Tea=Valeur_Tea;
		// Pour la prise d'chantillon
		 S_Echantillon_Acquis=S_Mesure_Position;
		 IB_Echantillon_Acquis=1;
		}
	}
}
//****************************************
//	FONCTION PRINCIPALE 	
//****************************************
 main()
{
  // INIT TPU   (Voir TP n1  et n2-1 )
  CPR0=0x0000;
  CFSR0=0x0888,CFSR1=0x8866;	
  Cor_PinS_CHA=0x0096,Edge_T_CHA=0x0081;	
  Cor_PinS_CHB=0x0086,Edge_T_CHB=0x0081;	
  HSQR0=0x0001,HSRR0=0x196F,CPR0=0x155F;	
// Pour Initialiser le timer programmable	
 SetVect(96,&Irq_Base_De_Temps);	// mise en place de l'autovecteur 
 PITR = 0x0008;		// Une interruption toutes les millisecondes 
 PICR = 0x0760; 		//  96 = 60H
// Pour Initialiser la liaison srie RS232
  SCCR0 = 9;			// Pour la vitesse de transmission de 57600 bauds
  SCCR1= 0x002C;		// Pour valider l'mission et la rceptionn srie 
// Pour Initialiser la commande moteur
Cde_Moteur=148;  // Commande moteur
// Pour Initialiser les capteurs de position et de vitesse
S_Mesure_Position=0;
S_Mesure_Position_Precedente=0;
S_Mesure_Vitesse=0;
Cptr_Te=Valeur_Te;
//  Pour Initialiser la consigne 
S_Consigne=0;
// Pour l'acquisition de la rponse temporelle
IB_Autorisation_Acquisition=0;
IB_Echantillon_Acquis=0;
IB_Emission_En_Cours=0;
C_Compteur_Echantillons=0;
Cptr_Tea=Valeur_Tea;
// Pour informer l'oprateur
	clsscr();
	gotoxy(4,6);
	printf("     Pour appliquer la consigne demarrer l'acquisition\n");
	gotoxy(4,8);
	printf("     -> Regler la position du potentiometre\n");
	gotoxy(4,10);
	printf("     -> Puis appuyer sur bouton poussoir CRTL\n");
// Pour la conversion
CAN=0x0043; // Pour lancer la premiere conversion
/*  DEBUT DE LA BOUCLE PRINCIPALE
----------------------------------*/
do
	{ 	if (Fin_Conv_AN==0)
			{S_Valeur_Pot=(short)(CAN&0x0FFC); // Sur 12bits et on masque les 2 bits LSB
		  	 CAN=0x0043; // Pour relancer la conversion de la voie 3 (Potentiometre)
			}
		if((BP_CRTL==0)&&(IB_Autorisation_Acquisition==0))
			{// On applique la consigne et on demarre l'acquisition 
			 clsscr();  // Pour effacer l'ecran
			 S_Mesure_Position=0; // Pour initialiser la mesure 
			 S_Mesure_Position_Precedente=0;
			 S_Consigne=S_Valeur_Pot>>2;
			 Cptr_Te=Valeur_Te;	
	 		 Cptr_Tea=Valeur_Tea;		 
			 sprintf(TC_Caracteres_A_Emettre,"%4d\n\r",S_Consigne);
			 C_Rang_Caractere_A_Emettre=0;
			  // La premiere donne transmise est la consigne position
			 IB_Emission_En_Cours=1;
		 	 IB_Autorisation_Acquisition=1;
			}
  		if(IB_Autorisation_Acquisition==0)
			{US_Compteur_Affichage++;
			 if(US_Compteur_Affichage==50000)
			 	{gotoxy(4,16);
			 	 printf("  Valeur consigne  %3.3d\n", S_Valeur_Pot>>2);
				}
			}
		// Si un echantillon a ete acquis
		if((IB_Echantillon_Acquis==1)&&(IB_Emission_En_Cours==0))
			{IB_Echantillon_Acquis=0;
			 sprintf(TC_Caracteres_A_Emettre,"%4d\n\r",S_Echantillon_Acquis);
			 C_Rang_Caractere_A_Emettre=0;
			 C_Compteur_Echantillons++;
			 if(C_Compteur_Echantillons>NB_Max_Echantillons)IB_Autorisation_Acquisition=0;
       		 IB_Emission_En_Cours=1;
			}
		
		if(IB_Emission_En_Cours==1)
			{if(SCSR&TDRE)  // Si Registre Emission libre
				{ // Charger le caractre  mettre
				 SCDR=TC_Caracteres_A_Emettre[C_Rang_Caractere_A_Emettre];	
			   	  if(C_Rang_Caractere_A_Emettre>=6) 
						{IB_Emission_En_Cours=0;		// On a fini d'mettre
						}
			  	  else C_Rang_Caractere_A_Emettre++;
				}
			}
	}while(C_Compteur_Echantillons<NB_Max_Echantillons);	/* fin de la boucle principale */
PICR=0;		// Arreter le Timer
RestoreVect(96);	// Pour les interruptions ... sortir proprement! 
Cde_Moteur=130;   // Remettre la commande du moteur  0
} // Fin de la fonction principale


