	/********************************************************************************
	*       TP sur  EID210 avec Interface pour Servo Mcanismes (ISM)		*
	*********************************************************************************
	*	Mesure numrique de la vitesse de rotation moteur			*
	*										*
	*   CAHIER DES CHARGES :					      		*
	*  ***************************					      		*	
	*   On souhaite que l'image de la tension aux bornes du moteur 		*
	*   suive  la tension impose par le potentiomtre analogique  		*
	*    		Tension U_Pot_Int sur EA3			 		*
	*	-> La conversion A->N de U_Pot est demande toutes les  mS	*
	*	-> Cette priode d'chantillonnage est impose par	le timer	*
	*	     programmable en mode d'interruption				*
	*	-> Les 8bits MSB du rsultat devienne la consigne de U		*
	*    On calcule un image numrique de la vitesse				*
	*      Image_vit = Image_pos actuelle - Image pos prcdente		*
 	*---------------------------------------------------------------------------------------------------------------------	*
	*  NOM du FICHIER :  Suivi_Consigne_Tension.C	       			*
	* ********************								*
	*********************************************************************************/
/* Liste des fichiers  inclure */
#include "Cpu_reg.h"
#include "EID210_reg.h"
#include "ISM.h"
#include "Structures_donnees.h"
#include <stdio.h>	// Pour pouvoir utiliser les fonctions de communication srie

/* Dclaration des variables globale */
WORD Val_Cde_Mot,i,Resultat_Conv,Pos_Mot_Prec;
signed char Im_Vit_Num;
signed short Im_Vit_W;

/* FONCTION d'interruption pour temporisation  (base de temps: toutes les mS)
=================================================================*/
void irq_bt()
{	// Acquisition et application consigne tension moteur 
	if (Fin_Conv_AN==1)
		{ Resultat_Conv=CAN;
		  Cons_TM=((Resultat_Conv&0x0FFF)>>4);	// Appliquer la Consigne Tension Moteur (Commande en PWM) 
		}
	// Calcul de l'image numrique de la vitesse
	Im_Vit_W = Pos_Mot - Pos_Mot_Prec;
	// Mise  l'chelle et application en sortie
	if(Im_Vit_W>127) Im_Vit_Num = 127;
	 else if(Im_Vit_W<-128) Im_Vit_Num = -128;
		else Im_Vit_Num =(signed char)(Im_Vit_W+128);
		Mes_IV= Im_Vit_Num;		// Actualiser la sortie analogique Image Vitesse	
	// Mise  jour des valeurs mmorises
	Pos_Mot_Prec = Pos_Mot;
  	CAN=0x0043; 	// Lancer la conversion de la voie n3 du CAN -> EA3 -> U_Pot
} // Fin de la dfinition de la fonction d'interruption pour la temporisation


/*=====================
  *    Fonction principale          
  =====================*/
main(void)
{

/* Initialisations */

/* Des registres du TPU pour configurer le bit PA0 -> ch0  (TPU7) en sortir PWM */	 
 TPUMCR=0x0040; 		// tcr1 
 CPR1=0x0000;	  	// Pendant init pas de spcification de priorit sur ch 0 7 
 CFSR3=0x0009;     		//  Rgistre de configuration pour Canaux 3..0  (PWM sur ch 0 donc sur PA0)
 CTRL_TPU7=0x0091;  	// On utilise TCR1  
 HSRR1=0x0002; 		// Initialisation canal	  
 CPR1=0x0003;      		// Priorit haute pour ch0 -> PA0
 Per_Cde_Mot=0x0100;   	//  	Valeur de la PERiode du signal				
   // Remarque:  Le Rapport Cyclique  (RC_Cde_Mot)  ne pourra varier que de 0x0000  0x00F00	
 Cons_TM=0x0000;   	// Dure du signal au niveau logique 1  Rapport cyclique initialis  0 %  
			// C'est la Consigne Tension Moteur	 	
/* Des registres du TPU pour configurer le comptage sur signaux en quadrature de phase issu du codeur moteur
// bit PB0 -> ch8 (TPU8) en canal primaire et bit PB1 -> ch9 (TPU9) en canal secondaire */	 
/* Effacer les bits de priorit	*/
   CPR0=0x0000;	
/* Registre de Slection de la Fonction des canaux d'E/S  11  10  9   8	*/ 
   CFSR1=0x0066;	/* les bit PB1 et PB0  en QDEC en comptage de signaux en quadrature)  	*/
/* Initialisation des paramtres RAM	 */
  Pos_Mot=0x0000;
  Cor_PinS_AD1_Mot=0x0026;
  Edge_T_LSB_AD1_Mot=0x0011;
  Cor_PinS_AD2_Mot=0x0016;
  Edge_T_LSB_AD2_Mot=0x0011;
/* Registre de squence de service	*/  
   HSQR0=0x0008;     	// Pour dfinir  PB1 secondaire -> 01   PB0 primaire ->  00
   HSRR0=0x000F; 	// Pour initialiser et dmarrer le comptage: pour  PB1 -> 11   et pour PB0 -> 11  
 /* Positionner les bits de priorit	*/
   CPR0=0x000F;	// Priorits maxi pour les 2 canaux PB1-PB0 
  
/* De l'interface de puissance de commande moteur */
 Dir_PortC=0xC2; 		// Signe_Cde_Mot et Valid_IP_Mot  seront actifs
 Valid_IP_Mot = 0; 		// Validation de l'Interface de Puissance moteur
 Signe_Cde_Mot=0;		// Commande Moteur en positif
  Val_Cod = 0;			// Pour Valider l'interface codeur
  Stb_Cod = 1 ;			// Pour valider le fonctionnement du codeur

/* Des variables diverses
Pos_Mot_Prec =0x0000;

/*   Boucle principale   */
/*************************/    
do	{  							// Rgulirement on envoie  l'cran:
	  printf("Valeur consigne: %x\n",Resultat_Conv);	// La tension potentiomtre convertie en numrique
	  printf("Image Vitesse: %x\n",Im_Vit_Num);		// La mesure numrique de la vitesse 
	  for(i=0;i<60000;i++);	// Attendre environ une seconde
	}while(1);
/* fin de la boucle principale */
} /* fin fonction principale */

