	/******************************************************************************
	*     TP sur  EID210 avec Interface pour Servo Mcanismes	(ISM)			*
	*******************************************************************************
	* Suivi de consigne donne par potentiomtre numrique pour Tension moteur	*
	*													*
	*  CAHIER DES CHARGES :					      				*
	*  *******************					      				*	
	*   On souhaite que l'image de la tension moteur soit impose par la position	*
	*   du potentiomtre numrique								*
	*	-> La position est chantillonne toutes les  mS				*
	*	-> Cette priode d'chantillonnage est impose par				*
	*	    le timer programmable en mode d'interruption				*
	*	-> Les 8bits LSB du rsultat devienne la consigne de I moteur		*
	*	-> La consigne courant est affiche en Hexa et en dcimal			*
 	*----------------------------------------------------------------------------	*
	*  NOM du FICHIER :  Suivi_Cons_PN_T_Mot.C	       				*
	* *****************										*
	*******************************************************************************/
/* Liste des fichiers  inclure */
#include "Cpu_reg.h"
#include "EID210_reg.h"
#include "Structures_donnees.h"
#include "ISM.h"
#include <stdio.h>



/* Dclaration des variables globale */
char Signe_Cde;
WORD i,Compteur_passages,Consigne_Tension;
signed short resultat,S_Temp,Valeur_acquise;

/* FONCTION d'interruption pour temporisation  (base de temps: toutes les mS)
=================================================================*/
void irq_bt()
{	 Valeur_acquise=S_Pot_Num;
} // Fin de la dfinition de la fonction d'interruption pour la temporisation

/*==========================
  *    Fonction principale          
  ==========================*/
main(void)
{
/* Initialisation des variables */

/* Les registres du TPU pour comptage Potentiomtre Numrique		
 ------------------------------------------------------------ */
/* Effacer les bits de priorit		*/
 CPR0=0x0000;	
/* Registre de Slection de la Fonction des canaux d'E/S  15  14  13   12	*/ 
 CFSR0=0x0066;	/* les bit PB4 (CH12) et PB5(CH13)  en QDEC en comptage de signaux en quadrature)  	*/
/* Initialisation des paramtres RAM	 */
// Pour le CH12  qui sera le canal primaire
 Cor_PinS_PNA=0x00D6;	//  D -> on compare par rapport au canal 13 ($D)   (fonction 6 / canal 13)
 Edge_T_LSB_PNA=0x00D1;//  Le rsultat de comptage est dans RAM canal 12 ($C)
// Pour CH13 qui sera le canal secondaire
 Cor_PinS_PNB=0x00C6;	//  C -> on compare par rapport au canal 12  ($C) (fonction 6 / canal 12)
 Edge_T_LSB_PNB=0x00D1;//  Le rsultat de comptage est dans RAM canal 12 ($C)
 S_Pot_Num=0x0000;	// Le compteur est initialis  0
/* Registre de squence de service	*/  
 HSQR0=0x0100;     	// Pour dfinir  PB4 secondaire -> 01   PB5 primaire ->  00
 HSRR0=0x0F00; 		// Pour initialiser et dmarrer le comptage: pour  PB5 -> 11   et pour PB4 -> 11  		
 /* Positionner les bits de priorit	*/
 CPR0=0x0F00;		// Priorits maxi pour les 2 canaux PB1-PB0 
/* Des registres du TPU pour configurer le bit PA0 -> ch0  (TPU0) en sortir PWM */	 
 TPUMCR=0x0040; 		// tcr1 
 CPR1=0x0000;	  	// pendant init pas de spcification de priorit sur ch 0 7 
 CFSR3=0x0009;     		//  Rgistre de configuration pour Canaux 3..0  (PWM sur ch 0 donc sur PA0)
 CTRL_TPU0=0x0091;  	// On utilise TCR1 
 HSRR1=0x0002; 		// Initialisation canal	  
 CPR1=0x0003;      	// Priorit haute pour ch0 -> PA0
 Per_Cde_Mot=0x00FF;   	//  	Valeur de la PERiode du signal				
   // Remarque:  Le Rapport Cyclique  (RC_Cde_Mot)  ne pourra varier que de 0x0000  0x00F00	
 Cons_TM=0x0000;   	// Dure du signal au niveau logique 1  Rapport cyclique initialis  0 %
 				// C'est la Consigne Tension Moteur

/* De l'interface de puissance de commande moteur */
 Dir_PortC = 0x03; 	// Signe_Cde_Mot et Valid_IP_Mot  seront actifs
 Valid_IP_Mot = 0; 	// Validation de l'Interface de Puissance moteur
 Signe_Cde_Mot = 0;	// On commande en positif (mettre 1 pour commander le moteur en ngatif)
 Cons_CM = 0x00FF;		// La consigne courant doit tre  son maximun
/* Pour affichage valeur consigne */ 
Compteur_passages = 0;
 
/* Pour le timer programmable	*/
 SetVect(96,&irq_bt);	// mise en place de l'autovecteur 
 PITR = 0x0008;		// Une interruption toutes les millisecondes 
 PICR = 0x0760; 		//  96 = 60H
  
/*   Boucle principale   */
/*************************/    
do	{resultat= Valeur_acquise>>3; 
	if (resultat>0)
		{Valid_IP_Mot=0; 	// On valide l'interface de puissance
		 Signe_Cde='+';
		 Signe_Cde_Mot=0;	// Commande du moteur en positif
	  	 if(resultat>255)Cons_TM=0x00FF,Consigne_Tension=255; // Le courant sera +Imax
	    	 else {Consigne_Tension=(WORD)(resultat);
			 Cons_TM=(WORD)(resultat);}}
	else if(resultat<0)
		{Valid_IP_Mot=0;	// On valide l'interface de puissance
		 Signe_Cde='-';
		 Signe_Cde_Mot=1;	// Commande du moteur en ngatif
	  	 if(resultat<-255)Cons_TM=0x00FF,Consigne_Tension=255; // Le courant sera -Imax
	    	 else {S_Temp=~resultat;
			 S_Temp+=1;
			 Consigne_Tension=(WORD)(S_Temp);
			 Cons_TM=(WORD)(S_Temp);}}
	else {Cons_TM=0x0000;
		Valid_IP_Mot=1;	 	// On inhibe l'interface de puissance
		Consigne_Tension=0;}	// Le courant sera nul
	// Pour affichage  intervalles de temps rguliers
	Compteur_passages++;
	if(Compteur_passages>50000)
		{Compteur_passages=0;
		 clsscr();
		 gotoxy(4,6);
	  	 printf("      AGIR SUR POTENTIOMETRE NUMERIQUE  \n");
 	  	 printf("         --------------------------------  \n");
	  	 gotoxy(4,10);
	  	 printf("Valeur numerique lue : %d (en Decimal)\n",resultat); // La consigne numrique 
	  	 gotoxy(4,12);
		 printf("Consigne module Tension Moteur = %2.2x (en Hexa) = %2d (en decimal)\n",Consigne_Tension,Consigne_Tension);
		 gotoxy(4,13);
		 printf("Avec la polarite  = %c\n",Signe_Cde);
		}
	}while(1);  /* Fin de la boucle principale */
} /* Fin fonction principale */

