/***************************************************************************
 * Module:        SMD_332.h						                           * 
 * Date:          Aout 2001			 		                               *
 * Auteur:        T. HANS  J.L. ROHOU                                      *
 							                                               *
 * Description:   Declarations pour carte CPU SMD332                       *
 **************************************************************************/

#ifndef _SMD332_H
#define _SMD332_H

/* Version materielle et logicielle */
/*==================================*/
#define VERSION_HARD	0x00	/* Version et revision du hard */
#define REVISION_HARD	0x00
#define VERSION_SOFT	0x00	/* Version et revision du programme */
#define	REVISION_SOFT	0x00

/* Redfinition des types */
		typedef unsigned char	BYTE;
		typedef unsigned short  WORD;
		typedef unsigned long	ULONG;

/* Dfinition des structures utiles */
/************************************/
/* Structure sur 8 bits 			*/
	union byte_bits
	{
		struct 
		 {
			unsigned char b7:1;
			unsigned char b6:1;
			unsigned char b5:1;
			unsigned char b4:1;
			unsigned char b3:1;
			unsigned char b2:1;
			unsigned char b1:1;
			unsigned char b0:1;
		}bit_b ;
		BYTE valeur_b;
	};
 /* Structure sur 16 bits */
	union word_bits
	{
		struct
		{
			unsigned char b15:1;
			unsigned char b14:1;
			unsigned char b13:1;
			unsigned char b12:1;
			unsigned char b11:1;
			unsigned char b10:1;
			unsigned char b9:1;
			unsigned char b8:1;
			unsigned char b7:1;
			unsigned char b6:1;
			unsigned char b5:1;
			unsigned char b4:1;
			unsigned char b3:1;
			unsigned char b2:1;
			unsigned char b1:1;
			unsigned char b0:1;
		} bit_w;
		WORD valeur_w;
	};

/* Structure sur 8 bits + octet*/
	union word_bits_octet
	{
		struct
		{
			unsigned char b15:1;
			unsigned char b14:1;
			unsigned char b13:1;
			unsigned char b12:1;
			unsigned char b11:1;
			unsigned char b10:1;
			unsigned char b9:1;
			unsigned char b8:1;
			unsigned char b_octet:8;
		} bits_octet;
		WORD val_wbo;
	};


 /* Adresses de bases des priphriques */
 #define	CTRL	   			  			0x900000 		/* 	CPLD de contrle       sur       CS2  	*/
 #define	REG_ETAT (*(union word_bits_octet*)	(CTRL+0x00)) 	/* registre d'tat (en lecture uniquement)	*/					
 #define	REG_CTRL 	(*(WORD*)			(CTRL+0x02))	/* rgistre de contrle (en lecture et ecr.	*/				 	
 #define  	Port_C		(*(union word_bits*)	(CTRL+0x100))	/*  Accs au port C                     */
 #define    DIR_Port_C	(*(WORD*)			(CTRL+0x102))	/* Accs au registre de direction du port C */                    
 #define	USB			 		  			0xB00000		/*  Ad. de base Port USB             CS3  	*/	   
 #define	CNA			 		  			0xB10000		/*  Ad. de base du Conv. Num -> Ana  CS4 	*/	   	
 #define	SA0			(*(BYTE*)	  	  	(CNA+0x00))		/*  	Sortie Analogique voie 0  (SA0)	 	*/
 #define	SA1			(*(BYTE*)	    	(CNA+0x02))		/*  	Sortie Analogique voie 0  (SA0)	 	*/
 #define	SA2			(*(BYTE*)	 	   	(CNA+0x04))		/*  	Sortie Analogique voie 0  (SA0)	 	*/
 #define	SA3			(*(BYTE*)	 	   	(CNA+0x06))		/*  	Sortie Analogique voie 0  (SA0)	 	*/
 #define	CAN			(*(WORD*)	 	   	0xB20000)		/*  Convertisseur Ana -> Num    sur  CS5  	*/
 #define	BUS			 		  			0xB30000		/*  Ad. de base du Conv. Num -> Ana  CS7 	*/		   
 
 /* Variables structures */
 /* Bits accessibles en lecture uniquement */
 #define Etat_reset		REG_ETAT.bits_octet.b15	/* RESERVE	 (Prog ne toune pas si RESET!		*/
 #define IRQ_CAN		REG_ETAT.bits_octet.b14	/* Bit de fin de conversion Ana -> Num 			*/
 #define IRQ_USB		REG_ETAT.bits_octet.b13	/* Bit d'tat de la ligne d'interruption USB	*/
 #define BUS_IRQ4		REG_ETAT.bits_octet.b12	/* Bit d'tat ligne IRQ4 du BUS					*/
 #define BUS_IRQ3		REG_ETAT.bits_octet.b11	/* Bit d'tat ligne IRQ3 du BUS					*/
 #define BUS_IRQ2		REG_ETAT.bits_octet.b10	/* Bit d'tat ligne IRQ2 du BUS					*/
 #define BUS_IRQ1		REG_ETAT.bits_octet.b9	/* Bit d'tat ligne IRQ1 du BUS					*/
 #define S_Controle		REG_ETAT.bits_octet.b8	/* Etat du "Switch de Contrle					*/
 #define N_VERSION		REG_ETAT.bits_octet.b_octet	/* N de VERSION soft PLD sur 8 bits		*/


  /* Bits accessibles en lecture et en criture */
 #define PC0		Port_C.bit_w.b8		  	/* Bit 0 du port C 		*/
 #define PC1		Port_C.bit_w.b9		 	/* Bit 1 du port C 		*/
 #define PC2		Port_C.bit_w.b10		/* Bit 2 du port C 		*/
 #define PC3		Port_C.bit_w.b11		/* Bit 3 du port C 		*/
 #define PC4		Port_C.bit_w.b12		/* Bit 4 du port C 		*/
 #define PC5		Port_C.bit_w.b13		/* Bit 5 du port C 		*/
 #define PC6		Port_C.bit_w.b14		/* Bit 6 du port C 		*/
 #define PC7		Port_C.bit_w.b15		/* Bit 7 du port C 		*/
 #define PC			Port_C.valeur_w			/* Port C sur 8Bits MSB */

#endif

