#include "eid210.h"
#include "d12ci.h"
#include "usb.h"

#define NUM_ENDPOINTS	4

#define CONFIG_DESCRIPTOR_LENGTH    sizeof(USB_CONFIGURATION_DESCRIPTOR) \
									+ sizeof(USB_INTERFACE_DESCRIPTOR) \
									+ (NUM_ENDPOINTS * sizeof(USB_ENDPOINT_DESCRIPTOR))


extern CONTROL_XFER ControlData;
extern USBFLAGS bUSBFlags;


USB_DEVICE_DESCRIPTOR DeviceDescr =
{
	18,
    USB_DEVICE_DESCRIPTOR_TYPE,
    0x0001,
    USB_CLASS_CODE_TEST_CLASS_DEVICE,
    0, 0,
	EP0_PACKET_SIZE,
	0x7104,
	0x1002,
	0x0100,
    0, 0, 0,
    1
};

char ConfDscr[] = {
			9,							// config Descripteur
			USB_CONFIGURATION_DESCRIPTOR_TYPE,
			46,0,
			1,
			1,
			0,
			0x60,
			1,
			9,							// Interface Descripteur
			USB_INTERFACE_DESCRIPTOR_TYPE,
    			0,
    			0,
			NUM_ENDPOINTS,
			USB_CLASS_CODE_TEST_CLASS_DEVICE,
			USB_SUBCLASS_CODE_TEST_CLASS_D12,
			USB_PROTOCOL_CODE_TEST_CLASS_D12,
			0,
			7,							// Endpoint TX1
			USB_ENDPOINT_DESCRIPTOR_TYPE,
			0x81,
			USB_ENDPOINT_TYPE_INTERRUPT,
			16,0,
			10,
			7,							// endpoint Rx1
			USB_ENDPOINT_DESCRIPTOR_TYPE,
			0x1,
			USB_ENDPOINT_TYPE_INTERRUPT,
			16,0,
			10,
			7,							// Endpoint Tx2
			USB_ENDPOINT_DESCRIPTOR_TYPE,
			0x82,
			USB_ENDPOINT_TYPE_BULK,
			64,0,
			10,
			7,							// Endpoint Rx2
			USB_ENDPOINT_DESCRIPTOR_TYPE,
			0x2,
			USB_ENDPOINT_TYPE_BULK,
			64,0,
			10
		      };

USB_CONFIGURATION_DESCRIPTOR ConfigDescr =
{
    9,
    USB_CONFIGURATION_DESCRIPTOR_TYPE,
    0x2e00,//CONFIG_DESCRIPTOR_LENGTH,
	1,
	1,
    0,
	0xA0,
	0x0
};

USB_INTERFACE_DESCRIPTOR InterfaceDescr =
{
	9,
    USB_INTERFACE_DESCRIPTOR_TYPE,
    0,
    0,
	NUM_ENDPOINTS,
	USB_CLASS_CODE_TEST_CLASS_DEVICE,
	USB_SUBCLASS_CODE_TEST_CLASS_D12,
	USB_PROTOCOL_CODE_TEST_CLASS_D12,
	0
};

USB_ENDPOINT_DESCRIPTOR EP1_TXDescr =
{
	7,
	USB_ENDPOINT_DESCRIPTOR_TYPE,
	0x81,
	USB_ENDPOINT_TYPE_INTERRUPT,
	EP1_PACKET_SIZE,
	10
};

USB_ENDPOINT_DESCRIPTOR EP1_RXDescr =
{
	7,
	USB_ENDPOINT_DESCRIPTOR_TYPE,
	0x1,
	USB_ENDPOINT_TYPE_INTERRUPT,
	EP1_PACKET_SIZE,
	10
};

USB_ENDPOINT_DESCRIPTOR EP2_TXDescr =
{
	7,
	USB_ENDPOINT_DESCRIPTOR_TYPE,
	0x82,
	USB_ENDPOINT_TYPE_BULK,
	EP2_PACKET_SIZE,
	10
};

USB_ENDPOINT_DESCRIPTOR EP2_RXDescr =
{
	7,
	USB_ENDPOINT_DESCRIPTOR_TYPE,
	0x2,
	USB_ENDPOINT_TYPE_BULK,
	EP2_PACKET_SIZE,
	10
};

/*
   //*************************************************************************
   // USB Protocol Layer
   //*************************************************************************
*/

void stall_ep0(void)
{
	D12_SetEndpointStatus(0, 1);
	D12_SetEndpointStatus(1, 1);
}

void reserved(void)
{
	stall_ep0();
}

void single_transmit(unsigned char * buf, unsigned char len)
{
	if( len <= EP0_PACKET_SIZE) {
		D12_WriteEndpoint(1, buf, len);
	}
}


void code_transmit(unsigned char * pRomData, unsigned short len)
{
	ControlData.wCount = 0;
	if(ControlData.wLength > len)
		ControlData.wLength = len;

	ControlData.pData = pRomData;
	if( ControlData.wLength >= EP0_PACKET_SIZE) {
		D12_WriteEndpoint(1, ControlData.pData, EP0_PACKET_SIZE);
		ControlData.wCount += EP0_PACKET_SIZE;

		bUSBFlags.bits.control_state = USB_TRANSMIT;
		
	}
	else {
		D12_WriteEndpoint(1, pRomData, ControlData.wLength);
		ControlData.wCount += ControlData.wLength;
		
		bUSBFlags.bits.control_state = USB_IDLE;
		
	}
}

void init_unconfig(void)
{
	unsigned char i;

	D12_SetEndpointEnable(0);	/* Disable all endpoints but EPP0. */
}

void init_config(void)
{
	D12_SetEndpointEnable(1);	/* Enable  generic/iso endpoints. */
}

/*
   //*************************************************************************
   // USB standard device requests
   //*************************************************************************
*/

void get_status(void)
{
	unsigned char endp, txdat[2];
	unsigned char bRecipient = ControlData.DeviceRequest.bmRequestType & USB_RECIPIENT;
	unsigned char c;

	if (bRecipient == USB_RECIPIENT_DEVICE) {
		if(bUSBFlags.bits.remote_wakeup == 1)
			txdat[0] = 3;
		else
			txdat[0] = 1;
		txdat[1]=0;
		single_transmit(txdat, 2);
	} else if (bRecipient == USB_RECIPIENT_INTERFACE) {
		txdat[0]=0;
		txdat[1]=0;
		single_transmit(txdat, 2);
	} else if (bRecipient == USB_RECIPIENT_ENDPOINT) {
		endp = (unsigned char)(ControlData.DeviceRequest.wIndex & MAX_ENDPOINTS);
		if (ControlData.DeviceRequest.wIndex & (unsigned char)USB_ENDPOINT_DIRECTION_MASK)
			c = D12_SelectEndpoint(endp*2 + 1);	/* Control-in */
		else
			c = D12_SelectEndpoint(endp*2);	/* Control-out */
		if(c & D12_STALL)
			txdat[0] = 1;
		else
			txdat[0] = 0;
		txdat[1] = 0;
		single_transmit(txdat, 2);
	} else
		stall_ep0();
}

void clear_feature(void)
{
	unsigned char endp;
	unsigned char bRecipient = ControlData.DeviceRequest.bmRequestType & USB_RECIPIENT;

	if (bRecipient == USB_RECIPIENT_DEVICE
		&& ControlData.DeviceRequest.wValue == USB_FEATURE_REMOTE_WAKEUP) {
		//DISABLE;
		bUSBFlags.bits.remote_wakeup = 0;
		//ENABLE;
		single_transmit(0, 0);
	}
	else if (bRecipient == USB_RECIPIENT_ENDPOINT
		&& ControlData.DeviceRequest.wValue == USB_FEATURE_ENDPOINT_STALL) {
		endp = (unsigned char)(ControlData.DeviceRequest.wIndex & MAX_ENDPOINTS);
		if (ControlData.DeviceRequest.wIndex & (unsigned char)USB_ENDPOINT_DIRECTION_MASK)
			/* clear TX stall for IN on EPn. */
			D12_SetEndpointStatus(endp*2 + 1, 0);
		else
			/* clear RX stall for OUT on EPn. */
			D12_SetEndpointStatus(endp*2, 0);
		single_transmit(0, 0);
	} else
		stall_ep0();
}

void set_feature(void)
{
	unsigned char endp;
	unsigned char bRecipient = ControlData.DeviceRequest.bmRequestType & USB_RECIPIENT;

	if (bRecipient == USB_RECIPIENT_DEVICE
		&& ControlData.DeviceRequest.wValue == USB_FEATURE_REMOTE_WAKEUP) {
		//DISABLE;
		bUSBFlags.bits.remote_wakeup = 1;
		//ENABLE;
		single_transmit(0, 0);
	}
	else if (bRecipient == USB_RECIPIENT_ENDPOINT
		&& ControlData.DeviceRequest.wValue == USB_FEATURE_ENDPOINT_STALL) {
		endp = (unsigned char)(ControlData.DeviceRequest.wIndex & MAX_ENDPOINTS);
		if (ControlData.DeviceRequest.wIndex & (unsigned char)USB_ENDPOINT_DIRECTION_MASK)
			/* clear TX stall for IN on EPn. */
			D12_SetEndpointStatus(endp*2 + 1, 1);
		else
			/* clear RX stall for OUT on EPn. */
			D12_SetEndpointStatus(endp*2, 1);
		single_transmit(0, 0);
	} else
		stall_ep0();
}

void set_address(void)
{
	D12_SetAddressEnable((unsigned char)(MSB(ControlData.DeviceRequest.wValue) &
		DEVICE_ADDRESS_MASK), 1);
	single_transmit(0, 0);
	printf("adresse=%x\n",MSB(ControlData.DeviceRequest.wValue)&DEVICE_ADDRESS_MASK);
}

void get_descriptor(void)
{
	unsigned char bDescriptor = ControlData.DeviceRequest.wValue;
	short length=MSB(ControlData.DeviceRequest.wLength);
printf("bDes=%x, len =%d\n",bDescriptor,length);
	if (bDescriptor == USB_DEVICE_DESCRIPTOR_TYPE) {
		code_transmit((unsigned char *)&DeviceDescr, sizeof(USB_DEVICE_DESCRIPTOR));
	} else if (bDescriptor == USB_CONFIGURATION_DESCRIPTOR_TYPE) {
		if (length==9)
			code_transmit(ConfDscr,9); // CONFIG_DESCRIPTOR_LENGTH);
			else
			code_transmit(ConfDscr,46);
			//code_transmit((unsigned char *)&InterfaceDescr ,37); //CONFIG_DESCRIPTOR_LENGTH-9);
		//code_transmit(ConfDscr,46);
	} else
		stall_ep0();
}

void get_configuration(void)
{
	unsigned char c = bUSBFlags.bits.configuration;

	single_transmit(&c, 1);
}

void set_configuration(void)
{
	if (ControlData.DeviceRequest.wValue == 0) {
		/* put device in unconfigured state */
		single_transmit(0, 0);
		//DISABLE;
		bUSBFlags.bits.configuration = 0;
		//ENABLE;
		init_unconfig();
	} else if (MSB(ControlData.DeviceRequest.wValue) == 1) {
		/* Configure device */
		single_transmit(0, 0);

		init_unconfig();
		init_config();
		
		//DISABLE;
		bUSBFlags.bits.configuration = 1;
		//ENABLE;
	} else
		stall_ep0();
}

void get_interface(void)
{
	unsigned char txdat = 0;        /* Only/Current interface = 0 */
	single_transmit(&txdat, 1);
}

void set_interface(void)
{
	if (ControlData.DeviceRequest.wValue == 0 && ControlData.DeviceRequest.wIndex == 0)
		single_transmit(0, 0);
	else
		stall_ep0();
}
