#include "eid210.h"
#include "d12ci.h"
#include "dms.h"
#include "usb_cdc.h"


#define NUM_ENDPOINTS	4

#define CONFIG_DESCRIPTOR_LENGTH    sizeof(USB_CONFIGURATION_DESCRIPTOR) \
									+ sizeof(USB_INTERFACE_DESCRIPTOR) \
									+ (NUM_ENDPOINTS * sizeof(USB_ENDPOINT_DESCRIPTOR))


extern CONTROL_XFER ControlData;
extern USBFLAGS bUSBFlags;


USB_DEVICE_DESCRIPTOR DeviceDescr =
{
	18,
    USB_DEVICE_DESCRIPTOR_TYPE,
    0x1001,
    0x02, // cdc driver
    0, 0,
	EP0_PACKET_SIZE,
	0x9c16,
	0x1002,
	0x0100,
    0, 0, 0,
    1
};

CDC_DESCRIPTOR cdc_dec={
{9,USB_CONFIGURATION_DESCRIPTOR_TYPE,0x4300,2,1,0,0x40,50},
{9,USB_INTERFACE_DESCRIPTOR_TYPE,0,0,1,0x02,0x02,0x01,0x00},
{0x05, 0x24, 0x00, 0x10, 0x01, 0x05, 0x24, 0x01, 0x03, 0x01, 0x04, 0x24, 0x02, 0x06,0x05, 0x24, 0x06, 0x00, 0x01},
{7,USB_ENDPOINT_DESCRIPTOR_TYPE,0x81,USB_ENDPOINT_TYPE_INTERRUPT,0x1000,10},
{9,USB_INTERFACE_DESCRIPTOR_TYPE,1,0,2,0x0a,0,0,0},
{7,USB_ENDPOINT_DESCRIPTOR_TYPE,0x82,USB_ENDPOINT_TYPE_BULK,0x4000,10},
{7,USB_ENDPOINT_DESCRIPTOR_TYPE,0x02,USB_ENDPOINT_TYPE_BULK,0x4000,10},
};

static char cdc_dec1[] ={
9,USB_CONFIGURATION_DESCRIPTOR_TYPE,0x43,00,2,1,0,0x40,50,
9,USB_INTERFACE_DESCRIPTOR_TYPE,0,0,1,0x02,0x02,0x01,0x00,
0x05, 0x24, 0x00, 0x10, 0x01, 0x05, 0x24, 0x01, 0x03, 0x01, 0x04, 0x24, 0x02, 0x06,0x05, 0x24, 0x06, 0x00, 0x01,
7,USB_ENDPOINT_DESCRIPTOR_TYPE,0x82,USB_ENDPOINT_TYPE_INTERRUPT,0x40,00,02,
9,USB_INTERFACE_DESCRIPTOR_TYPE,1,0,2,0x0a,0,0,0,
7,USB_ENDPOINT_DESCRIPTOR_TYPE,0x81,USB_ENDPOINT_TYPE_BULK,0x10,00,00,
7,USB_ENDPOINT_DESCRIPTOR_TYPE,0x01,USB_ENDPOINT_TYPE_BULK,0x10,00,00,
};

char line_coding[7];

/*
   //*************************************************************************
   // USB Protocol Layer
   //*************************************************************************
*/

void cdc_init()
{
  line_coding[0] = 0x00; // data terminal rate in bit per second - 4bytes
  line_coding[1] = 0xC2;
  line_coding[2] = 0x01;
  line_coding[3] = 00;
  line_coding[4] = 0; // stop bit
  line_coding[5] = 0; // parity
  line_coding[6] = 8; // data bits
}


void stall_ep0(void)
{
	D12_SetEndpointStatus(0, 1);
	D12_SetEndpointStatus(1, 1);
}

void reserved(void)
{
	stall_ep0();
}

void single_transmit(unsigned char * buf, unsigned char len)
{
	if( len <= EP0_PACKET_SIZE) {
		D12_WriteEndpoint(1, buf, len);
	}
}


void code_transmit(unsigned char * pRomData, unsigned short len)
{
	ControlData.wCount = 0;
	if(ControlData.wLength > len)
		ControlData.wLength = len;

	ControlData.pData = pRomData;
	if( ControlData.wLength >= EP0_PACKET_SIZE) {
		D12_WriteEndpoint(1, ControlData.pData, EP0_PACKET_SIZE);
		ControlData.wCount += EP0_PACKET_SIZE;

		bUSBFlags.bits.control_state = USB_TRANSMIT;
		
	}
	else {
		D12_WriteEndpoint(1, pRomData, ControlData.wLength);
		ControlData.wCount += ControlData.wLength;
		
		bUSBFlags.bits.control_state = USB_IDLE;
		
	}
}

void cdc_set_line_coding()
{
///printf("set line\n");
line_coding[0]=ControlData.dataBuffer[0];
line_coding[1]=ControlData.dataBuffer[1];
line_coding[2]=ControlData.dataBuffer[2];
line_coding[3]=ControlData.dataBuffer[3];
line_coding[4]=ControlData.dataBuffer[4];
line_coding[5]=ControlData.dataBuffer[5];
line_coding[6]=ControlData.dataBuffer[6];
single_transmit(0, 0);
//bUSBFlags.bits.control_state = USB_IDLE;
/*stall_ep0();*/
}

void cdc_get_line_coding()
{
//printf("get line\n");
single_transmit(line_coding,7);
//bUSBFlags.bits.control_state = USB_IDLE;

}


void init_unconfig(void)
{
	unsigned char i;

	D12_SetEndpointEnable(0);	/* Disable all endpoints but EPP0. */
}

void init_config(void)
{
	//printf("init\n");
	D12_SetEndpointEnable(1);	/* Enable  generic/iso endpoints. */
}

void usb_ep_init(void)
{

}

/*
   //*************************************************************************
   // USB standard device requests
   //*************************************************************************
*/

void get_status(void)
{
	unsigned char endp, txdat[2];
	unsigned char bRecipient = ControlData.DeviceRequest.bmRequestType & USB_RECIPIENT;
	unsigned char c;

	if (bRecipient == USB_RECIPIENT_DEVICE) {
		if(bUSBFlags.bits.remote_wakeup == 1)
			txdat[0] = 3;
		else
			txdat[0] = 1;
		txdat[1]=0;
		single_transmit(txdat, 2);
	} else if (bRecipient == USB_RECIPIENT_INTERFACE) {
		txdat[0]=0;
		txdat[1]=0;
		single_transmit(txdat, 2);
	} else if (bRecipient == USB_RECIPIENT_ENDPOINT) {
		endp = (unsigned char)(ControlData.DeviceRequest.wIndex & MAX_ENDPOINTS);
		if (ControlData.DeviceRequest.wIndex & (unsigned char)USB_ENDPOINT_DIRECTION_MASK)
			c = D12_SelectEndpoint(endp*2 + 1);	/* Control-in */
		else
			c = D12_SelectEndpoint(endp*2);	/* Control-out */
		if(c & D12_STALL)
			txdat[0] = 1;
		else
			txdat[0] = 0;
		txdat[1] = 0;
		single_transmit(txdat, 2);
	} else
		stall_ep0();
}

void clear_feature(void)
{
	unsigned char endp;
	unsigned char bRecipient = ControlData.DeviceRequest.bmRequestType & USB_RECIPIENT;

	if (bRecipient == USB_RECIPIENT_DEVICE
		&& ControlData.DeviceRequest.wValue == USB_FEATURE_REMOTE_WAKEUP) {
		//DISABLE;
		bUSBFlags.bits.remote_wakeup = 0;
		//ENABLE;
		single_transmit(0, 0);
	}
	else if (bRecipient == USB_RECIPIENT_ENDPOINT
		&& ControlData.DeviceRequest.wValue == USB_FEATURE_ENDPOINT_STALL) {
		endp = (unsigned char)(ControlData.DeviceRequest.wIndex & MAX_ENDPOINTS);
		if (ControlData.DeviceRequest.wIndex & (unsigned char)USB_ENDPOINT_DIRECTION_MASK)
			/* clear TX stall for IN on EPn. */
			D12_SetEndpointStatus(endp*2 + 1, 0);
		else
			/* clear RX stall for OUT on EPn. */
			D12_SetEndpointStatus(endp*2, 0);
		single_transmit(0, 0);
	} else
		stall_ep0();
}

void set_feature(void)
{
	unsigned char endp;
	unsigned char bRecipient = ControlData.DeviceRequest.bmRequestType & USB_RECIPIENT;

	if (bRecipient == USB_RECIPIENT_DEVICE
		&& ControlData.DeviceRequest.wValue == USB_FEATURE_REMOTE_WAKEUP) {
		//DISABLE;
		bUSBFlags.bits.remote_wakeup = 1;
		//ENABLE;
		single_transmit(0, 0);
	}
	else if (bRecipient == USB_RECIPIENT_ENDPOINT
		&& ControlData.DeviceRequest.wValue == USB_FEATURE_ENDPOINT_STALL) {
		endp = (unsigned char)(ControlData.DeviceRequest.wIndex & MAX_ENDPOINTS);
		if (ControlData.DeviceRequest.wIndex & (unsigned char)USB_ENDPOINT_DIRECTION_MASK)
			/* clear TX stall for IN on EPn. */
			D12_SetEndpointStatus(endp*2 + 1, 1);
		else
			/* clear RX stall for OUT on EPn. */
			D12_SetEndpointStatus(endp*2, 1);
		single_transmit(0, 0);
	} else
		stall_ep0();
}

void set_address(void)
{
	D12_SetAddressEnable((unsigned char)(MSB(ControlData.DeviceRequest.wValue) &
		DEVICE_ADDRESS_MASK), 1);
	single_transmit(0, 0);
	//printf("adresse=%x\n",MSB(ControlData.DeviceRequest.wValue)&DEVICE_ADDRESS_MASK);
}

void get_descriptor(void)
{
	unsigned char bDescriptor = ControlData.DeviceRequest.wValue;
	short length=ControlData.DeviceRequest.wLength;//+LSB(ControlData.DeviceRequest.wLength);
//printf("bDes=%x, len =%x\n",bDescriptor,length);
	if (bDescriptor == USB_DEVICE_DESCRIPTOR_TYPE) {
		code_transmit((unsigned char *)&DeviceDescr, sizeof(USB_DEVICE_DESCRIPTOR));
	} else if (bDescriptor == USB_CONFIGURATION_DESCRIPTOR_TYPE) {
		if (length==9)
			code_transmit((unsigned char *)&cdc_dec1,9); // CONFIG_DESCRIPTOR_LENGTH);
			else
			code_transmit((unsigned char *)&cdc_dec1,0x43);
			//code_transmit((unsigned char *)&InterfaceDescr ,37); //CONFIG_DESCRIPTOR_LENGTH-9);
		//code_transmit(ConfDscr,46);
	} else
		stall_ep0();
}

void get_configuration(void)
{
	unsigned char c = bUSBFlags.bits.configuration;

	single_transmit(&c, 1);
}

void set_configuration(void)
{
	if (ControlData.DeviceRequest.wValue == 0) {
		/* put device in unconfigured state */
		single_transmit(0, 0);
		//DISABLE;
		bUSBFlags.bits.configuration = 0;
		//ENABLE;
		init_unconfig();
	} else if (MSB(ControlData.DeviceRequest.wValue) == 1) {
		/* Configure device */
		single_transmit(0, 0);

		init_unconfig();
		init_config();
		
		//DISABLE;
		bUSBFlags.bits.configuration = 1;
		//ENABLE;
	} else
		stall_ep0();
}

void get_interface(void)
{
	unsigned char txdat = 0;        /* Only/Current interface = 0 */
	single_transmit(&txdat, 1);
}

void set_interface(void)
{
	if (ControlData.DeviceRequest.wValue == 0 && ControlData.DeviceRequest.wIndex == 0)
		single_transmit(0, 0);
	else
		stall_ep0();
}
